@@ -29,6 +29,7 @@ import java.util.Iterator;
 import java.util.regex.Matcher;
 import java.util.regex.Pattern;
 
+import javax.activation.DataSource;
 import javax.xml.parsers.SAXParser;
 import javax.xml.parsers.SAXParserFactory;
 
@@ -456,5 +457,180 @@ public class AttachmentDeserializerTest extends Assert {
         }
         return bout.toString();
     }
+    
+    @Test
+    public void testCXF3383() throws Exception {
+        String contentType = "multipart/related; type=\"application/xop+xml\";"
+            + " boundary=\"uuid:7a555f51-c9bb-4bd4-9929-706899e2f793\"; start=" 
+            + "\"<root.message@cxf.apache.org>\"; start-info=\"text/xml\"";
+        
+        Message message = new MessageImpl();
+        message.put(Message.CONTENT_TYPE, contentType);
+        message.setContent(InputStream.class, getClass().getResourceAsStream("cxf3383.data"));
+        message.put(AttachmentDeserializer.ATTACHMENT_DIRECTORY, System
+                .getProperty("java.io.tmpdir"));
+        message.put(AttachmentDeserializer.ATTACHMENT_MEMORY_THRESHOLD, String
+                .valueOf(AttachmentDeserializer.THRESHOLD));
+
+
+        AttachmentDeserializer ad 
+            = new AttachmentDeserializer(message, 
+                                         Collections.singletonList("multipart/related"));
+        
+        ad.initializeAttachments();
+        
+        
+        for (int x = 1; x < 50; x++) {
+            String cid = "1882f79d-e20a-4b36-a222-7a75518cf395-" + x + "@cxf.apache.org";
+            DataSource ds = AttachmentUtil.getAttachmentDataSource(cid, message.getAttachments());
+            byte bts[] = new byte[1024];
+            
+            InputStream ins = ds.getInputStream();
+            int count = ins.read(bts, 0, bts.length);
+            int sz = ins.read(bts, count, bts.length - count);
+            while (sz != -1) {
+                sz = ins.read(bts, count, bts.length - count);
+            }
+            assertEquals(x + 1, count);
+        }
+    }
+
+    
+    @Test
+    public void testCXF3582() throws Exception {
+        String contentType = "multipart/related; type=\"application/xop+xml\"; "
+            + "boundary=\"uuid:906fa67b-85f9-4ef5-8e3d-52416022d463\"; "
+            + "start=\"<root.message@cxf.apache.org>\"; start-info=\"text/xml\"";
+            
+            
+        Message message = new MessageImpl();
+        message.put(Message.CONTENT_TYPE, contentType);
+        message.setContent(InputStream.class, getClass().getResourceAsStream("cxf3582.data"));
+        message.put(AttachmentDeserializer.ATTACHMENT_DIRECTORY, System
+                .getProperty("java.io.tmpdir"));
+        message.put(AttachmentDeserializer.ATTACHMENT_MEMORY_THRESHOLD, String
+                .valueOf(AttachmentDeserializer.THRESHOLD));
+
+
+        AttachmentDeserializer ad 
+            = new AttachmentDeserializer(message, 
+                                         Collections.singletonList("multipart/related"));
+        
+        ad.initializeAttachments();
+        
+        String cid = "1a66bb35-67fc-4e89-9f33-48af417bf9fe-1@apache.org";
+        DataSource ds = AttachmentUtil.getAttachmentDataSource(cid, message.getAttachments());
+        byte bts[] = new byte[1024];
+        InputStream ins = ds.getInputStream();
+        int count = ins.read(bts, 0, bts.length);
+        assertEquals(500, count);
+        assertEquals(-1, ins.read(new byte[1000], 500, 500));
+
+        cid = "1a66bb35-67fc-4e89-9f33-48af417bf9fe-2@apache.org";
+        ds = AttachmentUtil.getAttachmentDataSource(cid, message.getAttachments());
+        bts = new byte[1024];
+        ins = ds.getInputStream();
+        count = ins.read(bts, 0, bts.length);
+        assertEquals(1024, count);
+        assertEquals(225, ins.read(new byte[1000], 500, 500));
+        assertEquals(-1, ins.read(new byte[1000], 500, 500));
+    }
+
+    @Test
+    public void testCXF3582b() throws Exception {
+        String contentType = "multipart/related; type=\"application/xop+xml\"; "
+            + "boundary=\"uuid:906fa67b-85f9-4ef5-8e3d-52416022d463\"; "
+            + "start=\"<root.message@cxf.apache.org>\"; start-info=\"text/xml\"";
+            
+            
+        Message message = new MessageImpl();
+        message.put(Message.CONTENT_TYPE, contentType);
+        message.setContent(InputStream.class, getClass().getResourceAsStream("cxf3582.data"));
+        message.put(AttachmentDeserializer.ATTACHMENT_DIRECTORY, System
+                .getProperty("java.io.tmpdir"));
+        message.put(AttachmentDeserializer.ATTACHMENT_MEMORY_THRESHOLD, String
+                .valueOf(AttachmentDeserializer.THRESHOLD));
+
+
+        AttachmentDeserializer ad 
+            = new AttachmentDeserializer(message, 
+                                         Collections.singletonList("multipart/related"));
+        
+        ad.initializeAttachments();
+        
+        String cid = "1a66bb35-67fc-4e89-9f33-48af417bf9fe-1@apache.org";
+        DataSource ds = AttachmentUtil.getAttachmentDataSource(cid, message.getAttachments());
+        byte bts[] = new byte[1024];
+        InputStream ins = ds.getInputStream();
+        int count = 0;
+        int x = ins.read(bts, 500, 200);
+        while (x != -1) {
+            count += x;
+            x = ins.read(bts, 500, 200);
+        }
+        assertEquals(500, count);
+        assertEquals(-1, ins.read(new byte[1000], 500, 500));
+
+        cid = "1a66bb35-67fc-4e89-9f33-48af417bf9fe-2@apache.org";
+        ds = AttachmentUtil.getAttachmentDataSource(cid, message.getAttachments());
+        bts = new byte[1024];
+        ins = ds.getInputStream();
+        count = 0;
+        x = ins.read(bts, 500, 200);
+        while (x != -1) {
+            count += x;
+            x = ins.read(bts, 500, 200);
+        }
+        assertEquals(1249, count);
+        assertEquals(-1, ins.read(new byte[1000], 500, 500));
+    }
+    @Test
+    public void testCXF3582c() throws Exception {
+        String contentType = "multipart/related; type=\"application/xop+xml\"; "
+            + "boundary=\"uuid:906fa67b-85f9-4ef5-8e3d-52416022d463\"; "
+            + "start=\"<root.message@cxf.apache.org>\"; start-info=\"text/xml\"";
+            
+            
+        Message message = new MessageImpl();
+        message.put(Message.CONTENT_TYPE, contentType);
+        message.setContent(InputStream.class, getClass().getResourceAsStream("cxf3582.data"));
+        message.put(AttachmentDeserializer.ATTACHMENT_DIRECTORY, System
+                .getProperty("java.io.tmpdir"));
+        message.put(AttachmentDeserializer.ATTACHMENT_MEMORY_THRESHOLD, String
+                .valueOf(AttachmentDeserializer.THRESHOLD));
+
+
+        AttachmentDeserializer ad 
+            = new AttachmentDeserializer(message, 
+                                         Collections.singletonList("multipart/related"));
+        
+        ad.initializeAttachments();
+        
+        String cid = "1a66bb35-67fc-4e89-9f33-48af417bf9fe-1@apache.org";
+        DataSource ds = AttachmentUtil.getAttachmentDataSource(cid, message.getAttachments());
+        byte bts[] = new byte[1024];
+        InputStream ins = ds.getInputStream();
+        int count = 0;
+        int x = ins.read(bts, 100, 600);
+        while (x != -1) {
+            count += x;
+            x = ins.read(bts, 100, 600);
+        }
+        assertEquals(500, count);
+        assertEquals(-1, ins.read(new byte[1000], 100, 600));
+
+        cid = "1a66bb35-67fc-4e89-9f33-48af417bf9fe-2@apache.org";
+        ds = AttachmentUtil.getAttachmentDataSource(cid, message.getAttachments());
+        bts = new byte[1024];
+        ins = ds.getInputStream();
+        count = 0;
+        x = ins.read(bts, 100, 600);
+        while (x != -1) {
+            count += x;
+            x = ins.read(bts, 100, 600);
+        }
+        assertEquals(1249, count);
+        assertEquals(-1, ins.read(new byte[1000], 100, 600));
+    }
+}
 
-}
\ No newline at end of file
