@@ -23,7 +23,6 @@ import java.io.OutputStream;
 import java.lang.annotation.Annotation;
 import java.lang.reflect.InvocationHandler;
 import java.lang.reflect.Method;
-import java.net.HttpURLConnection;
 import java.net.URI;
 import java.util.ArrayList;
 import java.util.Arrays;
@@ -47,6 +46,7 @@ import javax.xml.stream.XMLStreamWriter;
 import org.apache.cxf.common.i18n.BundleUtils;
 import org.apache.cxf.common.logging.LogUtils;
 import org.apache.cxf.endpoint.Endpoint;
+import org.apache.cxf.helpers.CastUtils;
 import org.apache.cxf.interceptor.AbstractOutDatabindingInterceptor;
 import org.apache.cxf.interceptor.Fault;
 import org.apache.cxf.interceptor.InterceptorProvider;
@@ -55,20 +55,20 @@ import org.apache.cxf.jaxrs.model.ClassResourceInfo;
 import org.apache.cxf.jaxrs.model.OperationResourceInfo;
 import org.apache.cxf.jaxrs.model.Parameter;
 import org.apache.cxf.jaxrs.model.ParameterType;
-import org.apache.cxf.jaxrs.model.URITemplate;
 import org.apache.cxf.jaxrs.provider.ProviderFactory;
 import org.apache.cxf.jaxrs.utils.FormUtils;
 import org.apache.cxf.jaxrs.utils.InjectionUtils;
+import org.apache.cxf.message.Exchange;
 import org.apache.cxf.message.Message;
 import org.apache.cxf.message.MessageContentsList;
 import org.apache.cxf.phase.Phase;
-import org.apache.cxf.transport.http.HTTPConduit;
 
 /**
  * Proxy-based client implementation
  *
  */
-public class ClientProxyImpl extends AbstractClient implements InvocationHandlerAware, InvocationHandler {
+public class ClientProxyImpl extends AbstractClient implements 
+    InvocationHandlerAware, InvocationHandler {
 
     private static final Logger LOG = LogUtils.getL7dLogger(ClientProxyImpl.class);
     private static final ResourceBundle BUNDLE = BundleUtils.getBundle(ClientProxyImpl.class);
@@ -77,7 +77,7 @@ public class ClientProxyImpl extends AbstractClient implements InvocationHandler
     private ClassResourceInfo cri;
     private boolean inheritHeaders;
     private boolean isRoot;
-    private Map<String, Object> valuesMap = Collections.emptyMap();
+    private Map<String, Object> valuesMap;
     
     public ClientProxyImpl(URI baseURI, ClassResourceInfo cri, boolean isRoot, 
                            boolean inheritHeaders, Object... varValues) {
@@ -98,23 +98,21 @@ public class ClientProxyImpl extends AbstractClient implements InvocationHandler
     }
     
     private void initValuesMap(Object... varValues) {
-        if (isRoot) {
-            List<String> vars = cri.getURITemplate().getVariables();
+        if (isRoot && varValues.length != 0) {
             valuesMap = new LinkedHashMap<String, Object>();
+            List<String> vars = cri.getURITemplate().getVariables();
             for (int i = 0; i < vars.size(); i++) {
-                if (varValues.length > 0) {
-                    if (i < varValues.length) {
-                        valuesMap.put(vars.get(i), varValues[i]);
-                    } else {
-                        org.apache.cxf.common.i18n.Message msg = new org.apache.cxf.common.i18n.Message(
-                             "ROOT_VARS_MISMATCH", BUNDLE, vars.size(), varValues.length);
-                        LOG.info(msg.toString());
-                        break;
-                    }
+                if (i < varValues.length) {
+                    valuesMap.put(vars.get(i), varValues[i]);
                 } else {
-                    valuesMap.put(vars.get(i), "");
+                    org.apache.cxf.common.i18n.Message msg = new org.apache.cxf.common.i18n.Message(
+                         "ROOT_VARS_MISMATCH", BUNDLE, vars.size(), varValues.length);
+                    LOG.info(msg.toString());
+                    break;
                 }
             }
+        } else {
+            valuesMap = Collections.emptyMap();
         }
     }
     
@@ -180,7 +178,14 @@ public class ClientProxyImpl extends AbstractClient implements InvocationHandler
         setRequestHeaders(headers, ori, types.containsKey(ParameterType.FORM), 
             bodyIndex == -1 ? null : params[bodyIndex].getClass(), m.getReturnType());
         
-        return doChainedInvocation(uri, headers, ori, params, bodyIndex, types, pathParams);
+        getState().setTemplates(getTemplateParametersMap(ori.getURITemplate(), pathParams));
+        
+        Object body = null;
+        boolean isForm = types.containsKey(ParameterType.FORM);
+        if (bodyIndex != -1 || isForm) {
+            body = isForm ? handleForm(types, params) : params[bodyIndex];
+        }
+        return doChainedInvocation(uri, headers, ori, body, bodyIndex, null, null);
         
     }
 
@@ -200,8 +205,7 @@ public class ClientProxyImpl extends AbstractClient implements InvocationHandler
         }
         for (Parameter p : parameters) {
             if (p.getType() == ParameterType.CONTEXT) {
-                // ignore
-                continue;
+                reportInvalidResourceMethod(ori.getMethodToInvoke(), "NO_CONTEXT_PARAMETERS");
             }
             map.add(p.getType(), p);
         }
@@ -284,7 +288,9 @@ public class ClientProxyImpl extends AbstractClient implements InvocationHandler
             if (formParams || bodyClass != null && MultivaluedMap.class.isAssignableFrom(bodyClass)) {
                 headers.putSingle(HttpHeaders.CONTENT_TYPE, MediaType.APPLICATION_FORM_URLENCODED);
             } else {
-                String cType = ori.getConsumeTypes().isEmpty() 
+                String cType = 
+                    bodyClass != null && InjectionUtils.isPrimitive(bodyClass) 
+                        ? MediaType.TEXT_PLAIN : ori.getConsumeTypes().isEmpty() 
                     || ori.getConsumeTypes().get(0).equals(MediaType.WILDCARD_TYPE) 
                     ? MediaType.APPLICATION_XML : ori.getConsumeTypes().get(0).toString();   
                 headers.putSingle(HttpHeaders.CONTENT_TYPE, cType);
@@ -293,18 +299,11 @@ public class ClientProxyImpl extends AbstractClient implements InvocationHandler
         
         List<MediaType> accepts = getAccept(headers);
         if (accepts == null) {
-            boolean produceWildcard = ori.getProduceTypes().size() == 0 
-                || ori.getProduceTypes().get(0).equals(MediaType.WILDCARD_TYPE);
-            if (produceWildcard) {
-                accepts = InjectionUtils.isPrimitive(responseClass)
-                    ? Collections.singletonList(MediaType.TEXT_PLAIN_TYPE)
-                    : Collections.singletonList(MediaType.APPLICATION_XML_TYPE);        
-            } else if (responseClass == Void.class) {
-                accepts = Collections.singletonList(MediaType.WILDCARD_TYPE);
-            } else {
-                accepts = ori.getProduceTypes();
-            }
-            
+            accepts = InjectionUtils.isPrimitive(responseClass) 
+                ? Collections.singletonList(MediaType.TEXT_PLAIN_TYPE)
+                : ori.getProduceTypes().size() == 0 
+                || ori.getProduceTypes().get(0).equals(MediaType.WILDCARD_TYPE) 
+                ? Collections.singletonList(MediaType.APPLICATION_XML_TYPE) : ori.getProduceTypes();
             for (MediaType mt : accepts) {
                 headers.add(HttpHeaders.ACCEPT, mt.toString());
             }
@@ -328,46 +327,40 @@ public class ClientProxyImpl extends AbstractClient implements InvocationHandler
     private List<Object> getPathParamValues(MultivaluedMap<ParameterType, Parameter> map,
                                             Object[] params,
                                             OperationResourceInfo ori) {
+        List<Parameter> paramsList =  getParameters(map, ParameterType.PATH);
         List<Object> list = new LinkedList<Object>();
         if (isRoot) {
             list.addAll(valuesMap.values());
         }
-        List<String> methodVars = ori.getURITemplate().getVariables();
-        
-        List<Parameter> paramsList =  getParameters(map, ParameterType.PATH);
-        Map<String, Parameter> paramsMap = new LinkedHashMap<String, Parameter>();
+        List<String> vars = ori.getURITemplate().getVariables();
+        // TODO : unfortunately, UriBuilder will lose a method-scoped parameter 
+        // if a same name variable exists in a class scope which is an api bug.
+        // It's a rare case but we might want just to use UriBuilderImpl() directly 
+        // on the client side and tell it to choose the last variable value
         for (Parameter p : paramsList) {
-            if (p.getName().length() == 0) {
-                MultivaluedMap<String, Object> values = 
-                    InjectionUtils.extractValuesFromBean(params[p.getIndex()], "");
-                for (String var : methodVars) {
-                    list.addAll(values.get(var));
-                }
-            } else {
-                paramsMap.put(p.getName(), p);
-            }
-        }
-        
-        for (String varName : methodVars) {
-            Parameter p = paramsMap.remove(varName);
-            if (p != null) {
-                list.add(params[p.getIndex()]);
-            }
-        }
-        
-        for (Parameter p : paramsMap.values()) {
-            if (valuesMap.containsKey(p.getName())) {
+            if (valuesMap.containsKey(p.getName()) && !vars.contains(p.getName())) {
                 int index = 0; 
                 for (Iterator<String> it = valuesMap.keySet().iterator(); it.hasNext(); index++) {
-                    if (it.next().equals(p.getName()) && index < list.size()) {
+                    if (it.next().equals(p.getName())) {
                         list.remove(index);
                         list.add(index, params[p.getIndex()]);
                         break;
                     }
                 }
-            }    
+            } else {
+                String paramName = p.getName();
+                if (!"".equals(paramName)) {
+                    list.add(params[p.getIndex()]);
+                } else {
+                    MultivaluedMap<String, Object> values = 
+                        InjectionUtils.extractValuesFromBean(params[p.getIndex()], "");
+                    for (String var : vars) {
+                        list.addAll(values.get(var));
+                    }
+                }
+                
+            }
         }
-        
         return list;
     }
     
@@ -443,63 +436,89 @@ public class ClientProxyImpl extends AbstractClient implements InvocationHandler
         }
     }
     
-    private Object doChainedInvocation(URI uri, MultivaluedMap<String, String> headers, 
-                          OperationResourceInfo ori, Object[] params, int bodyIndex, 
-                          MultivaluedMap<ParameterType, Parameter> types,
-                          List<Object> pathParams) throws Throwable {
-        Message outMessage = createMessage(ori.getHttpMethod(), headers, uri);
+    private Object doChainedInvocation(URI uri, 
+                                       MultivaluedMap<String, String> headers, 
+                                       OperationResourceInfo ori, 
+                                       Object body, 
+                                       int bodyIndex,
+                                       Exchange exchange,
+                                       Map<String, Object> invocationContext) throws Throwable {
+        
+        Message outMessage = createMessage(body, ori.getHttpMethod(), headers, uri, 
+                                           exchange, invocationContext);
+        
         outMessage.getExchange().setOneWay(ori.isOneway());
-        
-        getState().setTemplates(getTemplateParametersMap(ori.getURITemplate(), pathParams));
-        outMessage.put(URITemplate.TEMPLATE_PARAMETERS, getState().getTemplates());
-        
         outMessage.setContent(OperationResourceInfo.class, ori);
         setPlainOperationNameProperty(outMessage, ori.getMethodToInvoke().getName());
         outMessage.getExchange().put(Method.class, ori.getMethodToInvoke());
-        boolean isForm = types.containsKey(ParameterType.FORM);
-        if (bodyIndex != -1 || isForm) {
+        
+        if (body != null) {
             outMessage.put("BODY_INDEX", bodyIndex);
-            Object body = isForm ? handleForm(types, params) : params[bodyIndex];
-            MessageContentsList contents = new MessageContentsList(new Object[]{body});
-            outMessage.setContent(List.class, contents);
             outMessage.getInterceptorChain().add(new BodyWriter());
-        } else {
-            setEmptyRequestProperty(outMessage, ori.getHttpMethod());
         }
+
+        Map<String, Object> reqContext = getRequestContext(outMessage);
+        reqContext.put(OperationResourceInfo.class.getName(), ori);
+        reqContext.put("BODY_INDEX", bodyIndex);
         
         // execute chain    
         try {
             outMessage.getInterceptorChain().doIntercept(outMessage);
-        } catch (Throwable ex) {
-            // we'd like a user to get the whole Response anyway if needed
+        } catch (Exception ex) {
+            outMessage.setContent(Exception.class, ex);
         }
         
-        // TODO : this needs to be done in an inbound chain instead
-        HttpURLConnection connect = (HttpURLConnection)outMessage.get(HTTPConduit.KEY_HTTP_CONNECTION);
-        return handleResponse(connect, outMessage, ori);
+        Object[] results = preProcessResult(outMessage);
+        if (results != null && results.length == 1) {
+            // this can happen if a connection exception has occurred and
+            // failover feature used this client to invoke on a different address  
+            return results[0];
+        }
+        
+        Object response = null;
+        try {
+            response = handleResponse(outMessage);
+            return response;
+        } catch (Exception ex) {
+            response = ex;
+            throw ex;
+        } finally {
+            completeExchange(response, outMessage.getExchange());
+        }
         
     }
     
-    protected Object handleResponse(HttpURLConnection connect, Message outMessage, OperationResourceInfo ori) 
-        throws Throwable {
-        try {
-            Response r = setResponseBuilder(connect, outMessage.getExchange()).build();
-            Method method = ori.getMethodToInvoke();
-            checkResponse(method, r, outMessage);
-            if (method.getReturnType() == Void.class) { 
-                return null;
-            }
-            if (method.getReturnType() == Response.class
-                && (r.getEntity() == null || InputStream.class.isAssignableFrom(r.getEntity().getClass())
-                    && ((InputStream)r.getEntity()).available() == 0)) {
-                return r;
-            }
+    @Override
+    protected Object retryInvoke(URI newRequestURI, 
+                                 MultivaluedMap<String, String> headers,
+                                 Object body,
+                                 Exchange exchange, 
+                                 Map<String, Object> invContext) throws Throwable {
         
-            return readBody(r, outMessage, method.getReturnType(), 
-                        method.getGenericReturnType(), method.getDeclaredAnnotations());
-        } finally {
-            ProviderFactory.getInstance(outMessage).clearThreadLocalProxies();
+        Map<String, Object> reqContext = CastUtils.cast((Map)invContext.get(REQUEST_CONTEXT));
+        int bodyIndex = body != null ? (Integer)reqContext.get("BODY_INDEX") : -1;
+        OperationResourceInfo ori = 
+            (OperationResourceInfo)reqContext.get(OperationResourceInfo.class.getName());
+        return doChainedInvocation(newRequestURI, headers, ori, 
+                                   body, bodyIndex, exchange, invContext);
+    }
+    
+    protected Object handleResponse(Message outMessage) 
+        throws Throwable {
+        Response r = setResponseBuilder(outMessage, outMessage.getExchange()).build();
+        Method method = outMessage.getExchange().get(Method.class);
+        checkResponse(method, r, outMessage);
+        if (method.getReturnType() == Void.class) { 
+            return null;
         }
+        if (method.getReturnType() == Response.class
+            && (r.getEntity() == null || InputStream.class.isAssignableFrom(r.getEntity().getClass())
+                && ((InputStream)r.getEntity()).available() == 0)) {
+            return r;
+        }
+        
+        return readBody(r, outMessage, method.getReturnType(), 
+                        method.getGenericReturnType(), method.getDeclaredAnnotations());
     }
 
     public Object getInvocationHandler() {
@@ -564,6 +583,5 @@ public class ClientProxyImpl extends AbstractClient implements InvocationHandler
         }
         
     }
-
     
 }
