@@ -20,19 +20,19 @@
 package org.apache.cxf.jaxrs.impl;
 
 import java.util.Collections;
-import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
+import java.util.TreeMap;
 
 import javax.ws.rs.GET;
 import javax.ws.rs.Path;
 import javax.ws.rs.core.HttpHeaders;
+import javax.ws.rs.core.MediaType;
 import javax.ws.rs.core.Response;
 
 import org.apache.cxf.endpoint.Endpoint;
 import org.apache.cxf.jaxrs.JAXRSServiceImpl;
 import org.apache.cxf.jaxrs.model.ClassResourceInfo;
-import org.apache.cxf.jaxrs.model.wadl.WadlGenerator;
 import org.apache.cxf.jaxrs.provider.ProviderFactory;
 import org.apache.cxf.jaxrs.utils.ResourceUtils;
 import org.apache.cxf.message.Exchange;
@@ -42,8 +42,8 @@ import org.apache.cxf.message.MessageImpl;
 import org.apache.cxf.service.Service;
 import org.apache.cxf.service.model.EndpointInfo;
 import org.apache.cxf.transport.servlet.ServletDestination;
-import org.easymock.classextension.EasyMock;
-import org.easymock.classextension.IMocksControl;
+import org.easymock.EasyMock;
+import org.easymock.IMocksControl;
 
 import org.junit.Assert;
 import org.junit.Before;
@@ -87,7 +87,7 @@ public class RequestPreprocessorTest extends Assert {
         sqh.preprocess(m, new UriInfoImpl(m, null));
         Response r = m.getExchange().get(Response.class);
         assertNotNull(r);
-        assertEquals(WadlGenerator.WADL_TYPE.toString(),
+        assertEquals(MediaType.APPLICATION_XML,
                      r.getMetadata().getFirst(HttpHeaders.CONTENT_TYPE));
     }
     
@@ -120,6 +120,7 @@ public class RequestPreprocessorTest extends Assert {
         control.reset();
         Endpoint endp = control.createMock(Endpoint.class);
         e.put(Endpoint.class, endp);
+        EasyMock.expect(endp.isEmpty()).andReturn(true).anyTimes();
         endp.get(ProviderFactory.class.getName());
         EasyMock.expectLastCall().andReturn(ProviderFactory.getInstance()).anyTimes();
         ServletDestination d = control.createMock(ServletDestination.class);
@@ -131,7 +132,7 @@ public class RequestPreprocessorTest extends Assert {
         m.put(Message.REQUEST_URI, pathInfo);
         m.put(Message.QUERY_STRING, query);
         m.put(Message.HTTP_REQUEST_METHOD, method);
-        Map<String, List<String>> headers = new HashMap<String, List<String>>();
+        Map<String, List<String>> headers = new TreeMap<String, List<String>>(String.CASE_INSENSITIVE_ORDER);
         if (methodHeader != null) {
             headers.put("X-HTTP-Method-Override", Collections.singletonList(methodHeader));   
         }
