@@ -20,8 +20,11 @@ package org.apache.cxf.jaxrs.model.wadl;
 
 import java.io.StringReader;
 import java.util.ArrayList;
+import java.util.Arrays;
 import java.util.Collections;
+import java.util.HashSet;
 import java.util.List;
+import java.util.Set;
 
 import javax.ws.rs.core.HttpHeaders;
 import javax.ws.rs.core.MediaType;
@@ -42,9 +45,8 @@ import org.apache.cxf.message.MessageImpl;
 import org.apache.cxf.service.Service;
 import org.apache.cxf.service.model.EndpointInfo;
 import org.apache.cxf.transport.servlet.ServletDestination;
-import org.easymock.classextension.EasyMock;
-import org.easymock.classextension.IMocksControl;
-
+import org.easymock.EasyMock;
+import org.easymock.IMocksControl;
 import org.junit.Assert;
 import org.junit.Before;
 import org.junit.Test;
@@ -227,11 +229,15 @@ public class WadlGeneratorTest extends Assert {
                                                                  WadlGenerator.WADL_NS, "resource");
         assertEquals(1, resourceEls.size());        
         assertEquals("book", resourceEls.get(0).getAttribute("path"));
+        
+        verifyParameters(resourceEls.get(0), 1, new Param("id", "template", "xs:int"));
+        
+        checkGrammars(doc.getDocumentElement(), "thebook", null, "thechapter");
     }
     
     private void checkResponse(Response r) throws Exception {
         assertNotNull(r);
-        assertEquals(WadlGenerator.WADL_TYPE.toString(),
+        assertEquals(MediaType.APPLICATION_XML,
                      r.getMetadata().getFirst(HttpHeaders.CONTENT_TYPE));
 //        File f = new File("test.xml");
 //        f.delete();
@@ -246,6 +252,7 @@ public class WadlGeneratorTest extends Assert {
     @Test
     public void testMultipleRootResources() throws Exception {
         WadlGenerator wg = new WadlGenerator();
+        wg.setDefaultMediaType(WadlGenerator.WADL_TYPE.toString());
         ClassResourceInfo cri1 = 
             ResourceUtils.createClassResourceInfo(BookStore.class, BookStore.class, true, true);
         ClassResourceInfo cri2 = 
@@ -255,7 +262,8 @@ public class WadlGeneratorTest extends Assert {
         cris.add(cri2);
         Message m = mockMessage("http://localhost:8080/baz", "/bar", WadlGenerator.WADL_QUERY, cris);
         Response r = wg.handleRequest(m, null);
-        checkResponse(r);
+        assertEquals(WadlGenerator.WADL_TYPE.toString(),
+                     r.getMetadata().getFirst(HttpHeaders.CONTENT_TYPE));
         Document doc = DOMUtils.readXml(new StringReader(r.getEntity().toString()));
         checkGrammars(doc.getDocumentElement(), "thebook", "thebook2", "thechapter");
         List<Element> els = getWadlResourcesInfo(doc, "http://localhost:8080/baz", 2);
@@ -274,17 +282,24 @@ public class WadlGeneratorTest extends Assert {
         assertEquals("http://superbooks", schemasEls.get(0).getAttribute("targetNamespace"));
         List<Element> elementEls = DOMUtils.getChildrenWithName(schemasEls.get(0), 
                             XmlSchemaConstants.XSD_NAMESPACE_URI, "element");
-        assertEquals(3, elementEls.size());
+        
+        int size = book2El == null ? 2 : 3;
+        
+        assertEquals(size, elementEls.size());
         assertTrue(checkElement(elementEls, bookEl, "tns:book"));
-        assertTrue(checkElement(elementEls, book2El, "tns:book2"));
+        if (book2El != null) {
+            assertTrue(checkElement(elementEls, book2El, "tns:book2"));
+        }
         assertTrue(checkElement(elementEls, chapterEl, "tns:chapter"));
         
         List<Element> complexTypesEls = DOMUtils.getChildrenWithName(schemasEls.get(0), 
                                         XmlSchemaConstants.XSD_NAMESPACE_URI, "complexType");
-        assertEquals(3, complexTypesEls.size());
+        assertEquals(size, complexTypesEls.size());
         
         assertTrue(checkComplexType(complexTypesEls, "book"));
-        assertTrue(checkComplexType(complexTypesEls, "book2"));
+        if (book2El != null) {
+            assertTrue(checkComplexType(complexTypesEls, "book2"));
+        }
         assertTrue(checkComplexType(complexTypesEls, "chapter"));
     }
     
@@ -344,8 +359,8 @@ public class WadlGeneratorTest extends Assert {
         // must have a single template parameter
         verifyParameters(resource, 1, new Param("id", "template", "xs:long"));
         
-        // must have 2 methods, GET and PUT
-        List<Element> methodEls = getElements(resource, "method", 2);
+        // must have 3 methods, GET, POST and PUT
+        List<Element> methodEls = getElements(resource, "method", 3);
         
         // verify GET
         assertEquals("GET", methodEls.get(0).getAttribute("name"));
@@ -355,22 +370,28 @@ public class WadlGeneratorTest extends Assert {
         List<Element> requestEls = getElements(methodEls.get(0), "request", 1);
         
         // 4 parameters are expected
-        verifyParameters(requestEls.get(0), 4, 
+        verifyParameters(requestEls.get(0), 5, 
                          new Param("a", "query", "xs:int"),
                          new Param("c.a", "query", "xs:int"),
                          new Param("c.b", "query", "xs:int"),
-                         new Param("c.d.a", "query", "xs:int"));
+                         new Param("c.d.a", "query", "xs:int"),
+                         new Param("e", "query", "xs:string", Collections.singleton("A")));
         
         assertEquals(0, DOMUtils.getChildrenWithName(requestEls.get(0), 
                          WadlGenerator.WADL_NS, "representation").size());
         //check response
         verifyRepresentation(methodEls.get(0), "response", "text/plain", "");
         
-        // verify PUT
-        assertEquals("PUT", methodEls.get(1).getAttribute("name"));
-        verifyRepresentation(methodEls.get(1), "request", "text/plain", "");
+        // verify POST
+        assertEquals("POST", methodEls.get(1).getAttribute("name"));
+        Element formRep = verifyRepresentation(methodEls.get(1), "request", "multipart/form-data", "");
+        checkDocs(formRep, "", "Attachments", "");
         
-        verifyResponseWithStatus(methodEls.get(1), "204");
+        // verify PUT
+        assertEquals("PUT", methodEls.get(2).getAttribute("name"));
+        verifyRepresentation(methodEls.get(2), "request", "text/plain", "");
+        
+        verifyResponseWithStatus(methodEls.get(2), "204");
         
         // verify resource starting with /book2
         verifyGetResourceMethod(resourceEls.get(0), book2El, null);
@@ -392,9 +413,11 @@ public class WadlGeneratorTest extends Assert {
         
         checkDocs(requestEls.get(0), "", "Request", "");
         
-        verifyParameters(requestEls.get(0), 3, 
+        verifyParameters(requestEls.get(0), 4, 
                          new Param("hid", "header", "xs:int"),
                          new Param("provider.bar", "query", "xs:int"),
+                         new Param("bookstate", "query", "xs:string",
+                                 new HashSet<String>(Arrays.asList("NEW", "USED", "OLD"))),
                          new Param("a", "query", "xs:string", true));
         
         verifyXmlJsonRepresentations(requestEls.get(0), book2El, "InputBook");
@@ -422,13 +445,15 @@ public class WadlGeneratorTest extends Assert {
                          new Param("mid", "matrix", "xs:int"));
         checkDocs(resourceEls.get(5), "", "Book subresource", ""); 
         // should have 4 child resources
-        List<Element> subResourceEls = getElements(resourceEls.get(5), "resource", 4);
+        List<Element> subResourceEls = getElements(resourceEls.get(5), "resource", 6);
 
         assertEquals("/book", subResourceEls.get(0).getAttribute("path"));
         assertEquals("/form1", subResourceEls.get(1).getAttribute("path"));
         assertEquals("/form2", subResourceEls.get(2).getAttribute("path"));
-        assertEquals("/chapter/{cid}", subResourceEls.get(3).getAttribute("path"));
-        checkDocs(subResourceEls.get(3), "", "Chapter subresource", "");
+        assertEquals("/form3/{id}", subResourceEls.get(3).getAttribute("path"));
+        assertEquals("/form4/{id}", subResourceEls.get(4).getAttribute("path"));
+        assertEquals("/chapter/{cid}", subResourceEls.get(5).getAttribute("path"));
+        checkDocs(subResourceEls.get(5), "", "Chapter subresource", "");
         // verify book-subresource /book resource
         // GET 
         verifyGetResourceMethod(subResourceEls.get(0), bookEl, null);
@@ -453,9 +478,9 @@ public class WadlGeneratorTest extends Assert {
         
         
         // verify subresource /chapter/{id}
-        List<Element> chapterMethodEls = getElements(subResourceEls.get(3), "resource", 1);
+        List<Element> chapterMethodEls = getElements(subResourceEls.get(5), "resource", 1);
         assertEquals("/id", chapterMethodEls.get(0).getAttribute("path"));
-        verifyParameters(subResourceEls.get(3), 1, 
+        verifyParameters(subResourceEls.get(5), 1, 
                          new Param("cid", "template", "xs:int"));
         // GET
         verifyGetResourceMethod(chapterMethodEls.get(0), chapterEl, "Get the chapter");
@@ -513,7 +538,7 @@ public class WadlGeneratorTest extends Assert {
             WadlGenerator.WADL_NS, "representation").size());
     }
     
-    private void verifyRepresentation(Element element, 
+    private Element verifyRepresentation(Element element, 
                                       String name, 
                                       String mediaType,
                                       String elementValue) {
@@ -528,6 +553,7 @@ public class WadlGeneratorTest extends Assert {
             String pName = "request".equals(name) ? "request" : "result";
             verifyParameters(representationEls.get(0), 1, new Param(pName, "plain", "xs:string"));
         }
+        return representationEls.get(0);
     }
     
     private void verifyXmlJsonRepresentations(Element element, String type, String docs) {
@@ -552,6 +578,17 @@ public class WadlGeneratorTest extends Assert {
         assertEquals(p.getType(), paramEl.getAttribute("style"));
         assertEquals(p.getSchemaType(), paramEl.getAttribute("type"));
         assertEquals(p.isRepeating(), Boolean.valueOf(paramEl.getAttribute("repeating")));
+        Set<String> options = p.getOptions();
+        if (options != null) {
+            Set<String> actualOptions = new HashSet<String>();
+            List<Element> els = DOMUtils.getChildrenWithNamespace(paramEl, WadlGenerator.WADL_NS);
+            assertFalse(els.isEmpty());
+            assertEquals(options.size(), els.size());
+            for (Element op : els) {
+                actualOptions.add(op.getAttribute("value"));
+            }
+            assertEquals(options, actualOptions);
+        }
         String docs = p.getDocs();
         if (docs != null) {
             checkDocs(paramEl, "", docs, "");
@@ -602,10 +639,18 @@ public class WadlGeneratorTest extends Assert {
         private String schemaType;
         private String docs;
         private boolean repeating;
+        private Set<String> options;
         public Param(String name, String type, String schemaType) {
             this(name, type, schemaType, false);
         }
         
+        public Param(String name, String type, String schemaType, Set<String> opts) {
+            this.name = name;
+            this.type = type;
+            this.schemaType = schemaType;
+            this.options = opts;
+        }
+        
         public Param(String name, String type, String schemaType, boolean repeating) {
             this(name, type, schemaType, repeating, null);
         }
@@ -623,6 +668,10 @@ public class WadlGeneratorTest extends Assert {
             this.repeating = repeating;
         }
         
+        public Set<String> getOptions() {
+            return options;
+        }
+        
         public String getName() {
             return name;
         }
