@@ -22,6 +22,7 @@ package org.apache.cxf.jaxrs.provider;
 import java.io.ByteArrayInputStream;
 import java.io.ByteArrayOutputStream;
 import java.io.InputStream;
+import java.io.StringReader;
 import java.lang.annotation.Annotation;
 import java.lang.reflect.Method;
 import java.lang.reflect.ParameterizedType;
@@ -48,8 +49,12 @@ import javax.xml.bind.annotation.XmlElementWrapper;
 import javax.xml.bind.annotation.XmlMixed;
 import javax.xml.bind.annotation.XmlRootElement;
 import javax.xml.bind.annotation.XmlType;
+import javax.xml.stream.XMLStreamWriter;
 import javax.xml.transform.stream.StreamSource;
 
+import org.w3c.dom.Document;
+
+import org.apache.cxf.helpers.DOMUtils;
 import org.apache.cxf.jaxrs.fortest.jaxb.packageinfo.Book2;
 import org.apache.cxf.jaxrs.impl.MetadataMap;
 import org.apache.cxf.jaxrs.provider.JAXBElementProviderTest.TagVO2Holder;
@@ -60,6 +65,7 @@ import org.apache.cxf.jaxrs.resources.SuperBook;
 import org.apache.cxf.jaxrs.resources.TagVO;
 import org.apache.cxf.jaxrs.resources.TagVO2;
 import org.apache.cxf.jaxrs.resources.Tags;
+import org.apache.cxf.staxutils.StaxUtils;
 
 import org.junit.Assert;
 import org.junit.Ignore;
@@ -266,6 +272,62 @@ public class JSONProviderTest extends Assert {
         
     }
     
+    @SuppressWarnings("unchecked")
+    @Test
+    public void testWriteToSingleTag2NoNs() throws Exception {
+        JSONProvider p = new JSONProvider();
+        p.setIgnoreNamespaces(true);
+        TagVO2 tag = createTag2("a", "b");
+        
+        ByteArrayOutputStream os = new ByteArrayOutputStream();
+        
+        p.writeTo(tag, TagVO2.class, TagVO2.class, TagVO2.class.getAnnotations(), 
+                  MediaType.APPLICATION_JSON_TYPE, new MetadataMap<String, Object>(), os);
+        
+        String s = os.toString();
+        assertEquals("{\"thetag\":{\"group\":\"b\",\"name\":\"a\"}}", s);
+        
+    }
+    
+    @Test
+    public void testCopyReaderToDocument() throws Exception {
+        String s = "{\"tagVO\":{\"group\":\"b\",\"name\":\"a\"}}";
+        
+        ByteArrayInputStream is = new ByteArrayInputStream(s.getBytes());
+        
+        Document doc = (Document)new JSONProvider().readFrom((Class)Document.class, Document.class, 
+                  new Annotation[]{}, MediaType.APPLICATION_JSON_TYPE, 
+                  new MetadataMap<String, String>(), is);
+        ByteArrayOutputStream os = new ByteArrayOutputStream();
+        XMLStreamWriter writer = StaxUtils.createXMLStreamWriter(os);
+        StaxUtils.copy(doc, writer);
+        writer.writeEndDocument();
+        String s2 = os.toString();
+        assertTrue(s2.contains("<group>b</group><name>a</name>"));
+    }
+    
+    @Test
+    public void testWriteDocumentToWriter() throws Exception {
+        TagVO tag = createTag("a", "b");
+        
+        ByteArrayOutputStream os = new ByteArrayOutputStream();
+        
+        new JAXBElementProvider().writeTo(tag, (Class)TagVO.class, TagVO.class, 
+                  TagVO.class.getAnnotations(), MediaType.APPLICATION_XML_TYPE, 
+                  new MetadataMap<String, Object>(), os);
+        Document doc = DOMUtils.readXml(new StringReader(os.toString()));
+         
+        
+        ByteArrayOutputStream os2 = new ByteArrayOutputStream();
+        
+        new JSONProvider().writeTo(
+                  doc, Document.class, Document.class, 
+                  new Annotation[]{}, MediaType.APPLICATION_JSON_TYPE, 
+                  new MetadataMap<String, Object>(), os2);
+        String s = os2.toString();
+        assertEquals("{\"tagVO\":{\"group\":\"b\",\"name\":\"a\"}}", s);
+    }
+    
     @Test
     public void testWriteBookWithStringConverter() throws Exception {
         JSONProvider p = new JSONProvider();
@@ -335,6 +397,7 @@ public class JSONProviderTest extends Assert {
     public void testDropRootElement() throws Exception {
         JSONProvider p = new JSONProvider();
         p.setDropRootElement(true);
+        p.setIgnoreNamespaces(true);
         Map<String, String> namespaceMap = new HashMap<String, String>();
         namespaceMap.put("http://tags", "ns1");
         p.setNamespaceMap(namespaceMap);
@@ -354,24 +417,32 @@ public class JSONProviderTest extends Assert {
     public void testWriteQualifiedCollection() throws Exception {
         String data = "{\"ns1.tag\":[{\"group\":\"b\",\"name\":\"a\"}"
             + ",{\"group\":\"d\",\"name\":\"c\"}]}";
-        doWriteQualifiedCollection(false, false, data);
+        doWriteQualifiedCollection(false, false, false, data);
+    }
+    
+    @Test
+    public void testWriteQualifiedCollectionDropNs() throws Exception {
+        String data = "{\"tag\":[{\"group\":\"b\",\"name\":\"a\"}"
+            + ",{\"group\":\"d\",\"name\":\"c\"}]}";
+        doWriteQualifiedCollection(false, false, true, data);
     }
     
     @Test
     public void testWriteQualifiedCollection2() throws Exception {
         String data = "{{\"group\":\"b\",\"name\":\"a\"}"
             + ",{\"group\":\"d\",\"name\":\"c\"}}";
-        doWriteQualifiedCollection(true, false, data);
+        doWriteQualifiedCollection(true, false, false, data);
     }
     
     @Test
     public void testWriteQualifiedCollection3() throws Exception {
         String data = "[{\"group\":\"b\",\"name\":\"a\"}"
             + ",{\"group\":\"d\",\"name\":\"c\"}]";
-        doWriteQualifiedCollection(true, true, data);
+        doWriteQualifiedCollection(true, true, false, data);
     }
     
-    public void doWriteQualifiedCollection(boolean drop, boolean serializeAsArray, String data) 
+    public void doWriteQualifiedCollection(boolean drop, boolean serializeAsArray, 
+                                           boolean ignoreNamespaces, String data) 
         throws Exception {
         JSONProvider p = new JSONProvider();
         p.setCollectionWrapperName("{http://tags}tag");
@@ -380,6 +451,8 @@ public class JSONProviderTest extends Assert {
         Map<String, String> namespaceMap = new HashMap<String, String>();
         namespaceMap.put("http://tags", "ns1");
         p.setNamespaceMap(namespaceMap);
+        p.setIgnoreNamespaces(ignoreNamespaces);
+        
         List<TagVO2> tags = new ArrayList<TagVO2>();
         tags.add(createTag2("a", "b"));
         tags.add(createTag2("c", "d"));
@@ -456,7 +529,6 @@ public class JSONProviderTest extends Assert {
     }
     
     @Test
-    @Ignore("Enable once http://jira.codehaus.org/browse/JETTISON-104 gets resolved")
     public void testReadQualifiedArray() throws Exception {
         String data = "{\"ns1.thetag\":[{\"group\":\"b\",\"name\":\"a\"}"
             + ",{\"group\":\"d\",\"name\":\"c\"}]}";
@@ -513,6 +585,7 @@ public class JSONProviderTest extends Assert {
                   MediaType.APPLICATION_JSON_TYPE, new MetadataMap<String, Object>(), os);
         
         String s = os.toString();
+        System.out.println(s);
         assertEquals("{\"ns1.thetag\":{\"group\":\"b\",\"name\":\"a\"}}", s);
         
     }
@@ -530,9 +603,8 @@ public class JSONProviderTest extends Assert {
     }
     
     @Test
-    @Ignore("This is hitting http://jira.codehaus.org/browse/JETTISON-44")
     public void testWriteMixedContent() throws Exception {
-        doTestMixedContent("{\"Book\":{\"name\":\"CXF\",\"id\":125,\"$\":\"\\n     \\n\"}}",
+        doTestMixedContent("{\"Book\":{\"name\":\"CXF\",\"id\":125}}",
                            false, "book.xml");
     }
     
