@@ -27,6 +27,7 @@ import java.util.ArrayList;
 import java.util.Collections;
 import java.util.HashMap;
 import java.util.List;
+import java.util.Locale;
 import java.util.UUID;
 
 import javax.servlet.ServletConfig;
@@ -49,6 +50,7 @@ import javax.xml.bind.JAXBContext;
 
 import org.apache.cxf.endpoint.Endpoint;
 import org.apache.cxf.endpoint.Server;
+import org.apache.cxf.helpers.CastUtils;
 import org.apache.cxf.jaxrs.Customer;
 import org.apache.cxf.jaxrs.Customer2;
 import org.apache.cxf.jaxrs.CustomerApplication;
@@ -61,6 +63,7 @@ import org.apache.cxf.jaxrs.JAXRSServiceFactoryBean;
 import org.apache.cxf.jaxrs.JAXRSServiceImpl;
 import org.apache.cxf.jaxrs.SimpleFactory;
 import org.apache.cxf.jaxrs.Timezone;
+import org.apache.cxf.jaxrs.ext.ParameterHandler;
 import org.apache.cxf.jaxrs.impl.HttpHeadersImpl;
 import org.apache.cxf.jaxrs.impl.HttpServletResponseFilter;
 import org.apache.cxf.jaxrs.impl.MetadataMap;
@@ -87,6 +90,7 @@ import org.apache.cxf.message.Message;
 import org.apache.cxf.message.MessageImpl;
 import org.apache.cxf.transport.http.AbstractHTTPDestination;
 import org.easymock.EasyMock;
+
 import org.junit.Assert;
 import org.junit.Before;
 import org.junit.Test;
@@ -127,7 +131,7 @@ public class JAXRSUtilsTest extends Assert {
         assertEquals("2 form params should've been identified", 2, params.size());
         assertEquals("First Form Parameter not matched correctly",
                      "\u00E4\u00F6\u00FC", params.get(0));
-        List<String> list = (List<String>)params.get(1);
+        List<String> list = CastUtils.cast((List)params.get(1));
         assertEquals(2, list.size());
         assertEquals("2", list.get(0));
         assertEquals("3", list.get(1));
@@ -433,6 +437,15 @@ public class JAXRSUtilsTest extends Assert {
         
     }
     
+    @Test
+    public void testIntersectMimeTypesCompositeSubtype5() throws Exception {
+        List <MediaType> candidateList = 
+            JAXRSUtils.intersectMimeTypes("application/bar+xml", "application/bar+*");  
+
+        assertEquals(1, candidateList.size());
+        assertEquals("application/bar+xml", candidateList.get(0).toString());
+    }
+    
     @Test
     public void testIntersectMimeTypes() throws Exception {
         //test basic
@@ -673,14 +686,16 @@ public class JAXRSUtilsTest extends Assert {
     @SuppressWarnings("unchecked")
     @Test
     public void testQueryParamAsListWithDefaultValue() throws Exception {
-        Class[] argType = {List.class, List.class, List.class, List.class};
+        Class[] argType = {List.class, List.class, List.class, Integer[].class, 
+            List.class, List.class};
         Method m = Customer.class.getMethod("testQueryAsList", argType);
-        MessageImpl messageImpl = new MessageImpl();
-        messageImpl.put(Message.QUERY_STRING, "query2=query2Value&query3");
+        Message messageImpl = createMessage();
+        messageImpl.put(Message.QUERY_STRING, 
+                "query2=query2Value&query2=query2Value2&query3=1&query3=2&query4");
         List<Object> params = JAXRSUtils.processParameters(new OperationResourceInfo(m, null),
                                                            null, 
                                                            messageImpl);
-        assertEquals(4, params.size());
+        assertEquals(6, params.size());
         List<String> queryList = (List<String>)params.get(0);
         assertNotNull(queryList);
         assertEquals(1, queryList.size());
@@ -688,17 +703,30 @@ public class JAXRSUtilsTest extends Assert {
         
         List<String> queryList2 = (List<String>)params.get(1);
         assertNotNull(queryList2);
-        assertEquals(1, queryList2.size());
+        assertEquals(2, queryList2.size());
         assertEquals("query2Value", queryList2.get(0));
+        assertEquals("query2Value2", queryList2.get(1));
         
-        List<String> queryList3 = (List<String>)params.get(2);
+        List<Integer> queryList3 = (List<Integer>)params.get(2);
         assertNotNull(queryList3);
-        assertEquals(1, queryList3.size());
-        assertEquals("", queryList3.get(0));
+        assertEquals(2, queryList3.size());
+        assertEquals(Integer.valueOf(1), queryList3.get(0));
+        assertEquals(Integer.valueOf(2), queryList3.get(1));
         
-        List<String> queryList4 = (List<String>)params.get(3);
+        Integer[] queryList3Array = (Integer[])params.get(3);
+        assertNotNull(queryList3Array);
+        assertEquals(2, queryList3Array.length);
+        assertEquals(Integer.valueOf(1), queryList3Array[0]);
+        assertEquals(Integer.valueOf(2), queryList3Array[1]);
+        
+        List<String> queryList4 = (List<String>)params.get(4);
         assertNotNull(queryList4);
-        assertEquals(0, queryList4.size());
+        assertEquals(1, queryList4.size());
+        assertEquals("", queryList4.get(0));
+        
+        List<String> queryList5 = (List<String>)params.get(5);
+        assertNotNull(queryList5);
+        assertEquals(0, queryList5.size());
     }
     
     @Test
@@ -798,6 +826,24 @@ public class JAXRSUtilsTest extends Assert {
         
     }
     
+    @Test
+    public void testLocaleParameter() throws Exception {
+        Message messageImpl = createMessage();
+        ProviderFactory.getInstance(messageImpl).registerUserProvider(
+            new LocaleParameterHandler());
+        Class[] argType = {Locale.class};
+        Method m = Customer.class.getMethod("testLocaleParam", argType);
+        
+        messageImpl.put(Message.QUERY_STRING, "p1=en_us");
+        List<Object> params = JAXRSUtils.processParameters(new OperationResourceInfo(m, null),
+                                                           null, 
+                                                           messageImpl);
+        assertEquals(1, params.size());
+        Locale l = (Locale)params.get(0);
+        assertEquals("en", l.getLanguage());
+        assertEquals("US", l.getCountry());
+    }
+    
     @Test
     public void testConstructorFirstAndParameterHandler() throws Exception {
         Message messageImpl = createMessage();
@@ -1035,6 +1081,31 @@ public class JAXRSUtilsTest extends Assert {
 
         verifyParametersBean(m, null, messageImpl, null, complexMessageImpl);
     }
+    
+    @Test
+    public void testFormParametersBeanWithMap() throws Exception {
+        Class[] argType = {Customer.CustomerBean.class};
+        Method m = Customer.class.getMethod("testFormBean", argType);
+        Message messageImpl = createMessage();
+        messageImpl.put(Message.REQUEST_URI, "/bar");
+        MultivaluedMap<String, String> headers = new MetadataMap<String, String>();
+        headers.putSingle("Content-Type", MediaType.APPLICATION_FORM_URLENCODED);
+        messageImpl.put(Message.PROTOCOL_HEADERS, headers);
+        String body = "g.b=1&g.b=2";
+        messageImpl.setContent(InputStream.class, new ByteArrayInputStream(body.getBytes()));
+        List<Object> params = JAXRSUtils.processParameters(new OperationResourceInfo(m, null),
+                                                           null, 
+                                                           messageImpl);
+        assertEquals("Bean should be created", 1, params.size());
+        Customer.CustomerBean cb = (Customer.CustomerBean)params.get(0);
+        assertNotNull(cb);
+        assertNotNull(cb.getG());
+        List<String> values = cb.getG().get("b");
+        assertEquals(2, values.size());
+        assertEquals("1", values.get(0));
+        assertEquals("2", values.get(1));
+        
+    }
 
     private void verifyParametersBean(Method m,
                                       MultivaluedMap<String, String> simpleValues,
@@ -1713,4 +1784,13 @@ public class JAXRSUtilsTest extends Assert {
         e.put(Endpoint.class, endpoint);
         return m;
     }
+    
+    private static class LocaleParameterHandler implements ParameterHandler<Locale> {
+
+        public Locale fromString(String s) {
+            String[] values = s.split("_");
+            return values.length == 2 ? new Locale(values[0], values[1]) : new Locale(s);
+        }
+        
+    }
 }
