@@ -60,6 +60,8 @@ import org.apache.cxf.transport.ConduitInitiatorManager;
 import org.apache.cxf.transport.Destination;
 import org.apache.cxf.transport.MessageObserver;
 import org.apache.cxf.transport.http.AbstractHTTPDestination;
+import org.apache.cxf.transport.http.DestinationRegistry;
+import org.apache.cxf.transport.http.HTTPTransportFactory;
 import org.apache.cxf.transports.http.QueryHandler;
 import org.apache.cxf.transports.http.QueryHandlerRegistry;
 import org.apache.cxf.transports.http.StemMatchingQueryHandler;
@@ -69,7 +71,7 @@ import org.apache.cxf.ws.addressing.EndpointReferenceType;
 import org.apache.cxf.ws.addressing.JAXWSAConstants;
 import org.apache.cxf.ws.policy.PolicyEngine;
 import org.apache.cxf.wsdl.EndpointReferenceUtils;
-import org.easymock.classextension.EasyMock;
+import org.easymock.EasyMock;
 import org.eclipse.jetty.http.HttpFields;
 import org.eclipse.jetty.server.Request;
 import org.eclipse.jetty.server.Response;
@@ -109,7 +111,7 @@ public class JettyHTTPDestinationTest extends Assert {
     private QueryHandler wsdlQueryHandler;
     private QueryHandlerRegistry  queryHandlerRegistry;
     private List<QueryHandler> queryHandlerList;
-    private JettyHTTPTransportFactory transportFactory; 
+    private HTTPTransportFactory transportFactory; 
 
     /**
      * This class replaces the engine in the Jetty Destination.
@@ -117,13 +119,12 @@ public class JettyHTTPDestinationTest extends Assert {
     private class EasyMockJettyHTTPDestination
         extends JettyHTTPDestination {
 
-        public EasyMockJettyHTTPDestination(
-                Bus                       b,
-                JettyHTTPTransportFactory ci, 
-                EndpointInfo              endpointInfo,
-                JettyHTTPServerEngine     easyMockEngine
-        ) throws IOException {
-            super(b, ci, endpointInfo);
+        public EasyMockJettyHTTPDestination(Bus bus,
+                                            DestinationRegistry registry,
+                                            EndpointInfo endpointInfo,
+                                            JettyHTTPServerEngineFactory serverEngineFactory,
+                                            JettyHTTPServerEngine easyMockEngine) throws IOException {
+            super(bus, registry, endpointInfo, serverEngineFactory);
             engine = easyMockEngine;
         }
         
@@ -169,8 +170,8 @@ public class JettyHTTPDestinationTest extends Assert {
     
     @Test
     public void testRandomPortAllocation() throws Exception {
-        transportFactory = new JettyHTTPTransportFactory();
-        transportFactory.setBus(new CXFBusImpl());
+        transportFactory = new HTTPTransportFactory();
+        transportFactory.setBus(BusFactory.getDefaultBus());
         ServiceInfo serviceInfo = new ServiceInfo();
         serviceInfo.setName(new QName("bla", "Service"));
         EndpointInfo ei = new EndpointInfo(serviceInfo, "");
@@ -221,15 +222,16 @@ public class JettyHTTPDestinationTest extends Assert {
                 return httpEngine;
             }
         };
-        transportFactory = new JettyHTTPTransportFactory();
+        transportFactory = new HTTPTransportFactory();
         transportFactory.setBus(new CXFBusImpl());
         transportFactory.getBus().setExtension(
             factory, JettyHTTPServerEngineFactory.class);
         
-        
         TestJettyDestination testDestination = 
             new TestJettyDestination(transportFactory.getBus(), 
-                                     transportFactory, ei);
+                                     transportFactory.getRegistry(), 
+                                     ei,
+                                     factory);
         testDestination.finalizeConfig();
         Message mi = testDestination.retrieveFromContinuation(httpRequest);
         assertNull("Continuations must be ignored", mi);
@@ -237,8 +239,8 @@ public class JettyHTTPDestinationTest extends Assert {
     
     @Test
     public void testGetMultiple() throws Exception {
-        transportFactory = new JettyHTTPTransportFactory();
-        transportFactory.setBus(new CXFBusImpl());
+        transportFactory = new HTTPTransportFactory();
+        transportFactory.setBus(BusFactory.getDefaultBus(true));
         
         ServiceInfo serviceInfo = new ServiceInfo();
         serviceInfo.setName(new QName("bla", "Service"));        
@@ -248,11 +250,14 @@ public class JettyHTTPDestinationTest extends Assert {
         Destination d1 = transportFactory.getDestination(ei);
         
         Destination d2 = transportFactory.getDestination(ei);
+        
+        // Second get should not generate a new destination. It should just retrieve the existing one
         assertEquals(d1, d2);
         
         d2.shutdown();
         
         Destination d3 = transportFactory.getDestination(ei);
+        // Now a new destination should have been created
         assertNotSame(d1, d3);
     }
     
@@ -427,7 +432,7 @@ public class JettyHTTPDestinationTest extends Assert {
         address = getEPR("bar/foo");
         bus = new CXFBusImpl();
         
-        transportFactory = new JettyHTTPTransportFactory();
+        transportFactory = new HTTPTransportFactory();
         transportFactory.setBus(bus);
         
         ServiceInfo serviceInfo = new ServiceInfo();
@@ -442,7 +447,7 @@ public class JettyHTTPDestinationTest extends Assert {
         
         JettyHTTPDestination dest = 
             new EasyMockJettyHTTPDestination(
-                    bus, transportFactory, endpointInfo, engine);
+                    bus, transportFactory.getRegistry(), endpointInfo, null, engine);
         assertEquals(policy, dest.getServer());
     }
         
@@ -522,16 +527,7 @@ public class JettyHTTPDestinationTest extends Assert {
         address = getEPR("bar/foo");
         
 
-        transportFactory = new JettyHTTPTransportFactory() {
-            JettyHTTPServerEngineFactory serverEngineFactory;
-            @Override
-            public JettyHTTPServerEngineFactory getJettyHTTPServerEngineFactory() {
-                if (serverEngineFactory == null) {
-                    serverEngineFactory = new JettyHTTPServerEngineFactory();
-                }
-                return serverEngineFactory;   
-            }
-        };
+        transportFactory = new HTTPTransportFactory();
 
         final ConduitInitiator ci = new ConduitInitiator() {
             public Conduit getConduit(EndpointInfo targetInfo) throws IOException {
@@ -575,8 +571,10 @@ public class JettyHTTPDestinationTest extends Assert {
             bus = EasyMock.createMock(Bus.class);
             bus.getExtension(EndpointResolverRegistry.class);
             EasyMock.expectLastCall().andReturn(null);
-            bus.getExtension(PolicyEngine.class);
+            bus.getExtension(PolicyEngine.class);          
             EasyMock.expectLastCall().andReturn(null);
+            bus.getExtension(ClassLoader.class);
+            EasyMock.expectLastCall().andReturn(this.getClass().getClassLoader());
             EasyMock.replay(bus);
         }
         
@@ -597,8 +595,9 @@ public class JettyHTTPDestinationTest extends Assert {
         EasyMock.replay(engine);
         
         JettyHTTPDestination dest = new EasyMockJettyHTTPDestination(bus,
-                                                             transportFactory,
+                                                             transportFactory.getRegistry(),
                                                              endpointInfo,
+                                                             null,
                                                              engine);
         dest.retrieveEngine();
         policy = dest.getServer();
@@ -675,6 +674,8 @@ public class JettyHTTPDestinationTest extends Assert {
         EasyMock.expectLastCall().andReturn(method).atLeastOnce();
         request.getConnection();
         EasyMock.expectLastCall().andReturn(null).anyTimes();
+        request.getUserPrincipal();
+        EasyMock.expectLastCall().andReturn(null).anyTimes();
         
         if (setRedirectURL) {
             policy.setRedirectURL(NOWHERE + "foo/bar");
@@ -701,7 +702,8 @@ public class JettyHTTPDestinationTest extends Assert {
                 EasyMock.expect(request.getServletPath()).andReturn("");
                 EasyMock.expect(request.getPathInfo()).andReturn("/foo");
                 EasyMock.expect(request.getRequestURI()).andReturn("/foo");
-                EasyMock.expect(request.getRequestURL()).andReturn(new StringBuffer("http://test/request"));
+                EasyMock.expect(request.getRequestURL())
+                    .andReturn(new StringBuffer("http://localhost/foo")).anyTimes();
                 EasyMock.expect(request.getCharacterEncoding()).andReturn("UTF-8");
                 EasyMock.expect(request.getQueryString()).andReturn(query);    
                 EasyMock.expect(request.getHeader("Accept")).andReturn("*/*");  
@@ -751,16 +753,20 @@ public class JettyHTTPDestinationTest extends Assert {
         }
         
         if (decoupled) {
-            decoupledBackChannel = EasyMock.createMock(Conduit.class);
-            decoupledBackChannel.setMessageObserver(EasyMock.isA(MessageObserver.class));           
-            decoupledBackChannel.prepare(EasyMock.isA(Message.class));
-            EasyMock.expectLastCall();
-            EasyMock.replay(decoupledBackChannel);
+            setupDecoupledBackChannel();
         }
         EasyMock.replay(response);
         EasyMock.replay(request);
     }
     
+    private void setupDecoupledBackChannel() throws IOException {
+        decoupledBackChannel = EasyMock.createMock(Conduit.class);
+        decoupledBackChannel.setMessageObserver(EasyMock.isA(MessageObserver.class));           
+        decoupledBackChannel.prepare(EasyMock.isA(Message.class));
+        EasyMock.expectLastCall();
+        EasyMock.replay(decoupledBackChannel);
+    }
+    
     private void setUpInMessage() {
         inMessage.setExchange(new ExchangeImpl());
     }
@@ -957,10 +963,11 @@ public class JettyHTTPDestinationTest extends Assert {
     }
     
     private static class TestJettyDestination extends JettyHTTPDestination {
-        public TestJettyDestination(Bus b,
-                                    JettyHTTPTransportFactory ci, 
-                                    EndpointInfo endpointInfo) throws IOException {
-            super(b, ci, endpointInfo);
+        public TestJettyDestination(Bus bus,
+                                    DestinationRegistry registry,
+                                    EndpointInfo endpointInfo,
+                                    JettyHTTPServerEngineFactory serverEngineFactory) throws IOException {
+            super(bus, registry, endpointInfo, serverEngineFactory);
         }
         
         @Override
