@@ -37,6 +37,7 @@ import java.util.List;
 import java.util.Map;
 import java.util.Set;
 import java.util.concurrent.Executor;
+import java.util.concurrent.RejectedExecutionException;
 import java.util.logging.Level;
 import java.util.logging.Logger;
 
@@ -50,6 +51,8 @@ import org.apache.cxf.configuration.jsse.TLSClientParameters;
 import org.apache.cxf.configuration.security.AuthorizationPolicy;
 import org.apache.cxf.configuration.security.CertificateConstraintsType;
 import org.apache.cxf.configuration.security.ProxyAuthorizationPolicy;
+import org.apache.cxf.endpoint.ClientCallback;
+import org.apache.cxf.endpoint.Endpoint;
 import org.apache.cxf.helpers.HttpHeaderHelper;
 import org.apache.cxf.helpers.IOUtils;
 import org.apache.cxf.helpers.LoadingByteArrayOutputStream;
@@ -59,6 +62,7 @@ import org.apache.cxf.io.CachedOutputStream;
 import org.apache.cxf.message.Exchange;
 import org.apache.cxf.message.ExchangeImpl;
 import org.apache.cxf.message.Message;
+import org.apache.cxf.message.MessageContentsList;
 import org.apache.cxf.message.MessageImpl;
 import org.apache.cxf.message.MessageUtils;
 import org.apache.cxf.phase.PhaseInterceptorChain;
@@ -76,6 +80,7 @@ import org.apache.cxf.transport.https.CertConstraintsInterceptor;
 import org.apache.cxf.transport.https.CertConstraintsJaxBUtils;
 import org.apache.cxf.transport.https.HttpsURLConnectionFactory;
 import org.apache.cxf.transports.http.configuration.HTTPClientPolicy;
+import org.apache.cxf.workqueue.AutomaticWorkQueue;
 import org.apache.cxf.workqueue.WorkQueueManager;
 import org.apache.cxf.ws.addressing.EndpointReferenceType;
 import org.apache.cxf.ws.policy.Assertor;
@@ -173,6 +178,9 @@ public class HTTPConduit
      */
     private static final String SC_HTTP_CONDUIT_SUFFIX = ".http-conduit";
 
+    private static final String HTTP_POST_METHOD = "POST"; 
+    private static final String HTTP_PUT_METHOD = "PUT";
+    
     /**
      * This field holds the connection factory, which primarily is used to 
      * factor out SSL specific code from this implementation.
@@ -409,6 +417,7 @@ public class HTTPConduit
                                           getClient().getDecoupledEndpoint());
         }
         if (clientSidePolicy != null) {
+            clientSidePolicy.removePropertyChangeListener(this); //make sure we aren't added twice
             clientSidePolicy.addPropertyChangeListener(this);
         }
     }
@@ -507,8 +516,8 @@ public class HTTPConduit
         // DELETE does not work and empty PUTs cause misleading exceptions
         // if chunking is enabled
         // TODO : ensure chunking can be enabled for non-empty PUTs - if requested
-        if (connection.getRequestMethod().equals("POST")
-            && csPolicy.isAllowChunking()) {
+        if (csPolicy.isAllowChunking() 
+            && isChunkingSupported(message, connection.getRequestMethod())) {
             //TODO: The chunking mode be configured or at least some
             // documented client constant.
             //use -1 and allow the URL connection to pick a default value
@@ -536,17 +545,43 @@ public class HTTPConduit
 
         setHeadersByAuthorizationPolicy(message, currentURL);
         new Headers(message).setFromClientPolicy(getClient(message));
-
         message.setContent(OutputStream.class, 
-                           new WrappedOutputStream(
-                                   message, connection,
-                                   needToCacheRequest, 
-                                   isChunking,
-                                   chunkThreshold,
-                                   getConduitName()));
+                           createOutputStream(message, connection,
+                                              needToCacheRequest, 
+                                              isChunking,
+                                              chunkThreshold));
         // We are now "ready" to "send" the message. 
     }
 
+    protected boolean isChunkingSupported(Message message, String httpMethod) {
+        if (HTTP_POST_METHOD.equals(httpMethod)) { 
+            return true;
+        }
+        if (HTTP_PUT_METHOD.equals(httpMethod)) {
+            MessageContentsList objs = MessageContentsList.getContentsList(message);
+            if (objs != null && objs.size() > 0) {
+                Object obj = objs.get(0);
+                if (obj.getClass() != String.class 
+                    || (obj.getClass() == String.class && ((String)obj).length() > 0)) {
+                    return true;
+                }
+            }
+        }
+        return false;
+    }
+    
+    protected OutputStream createOutputStream(Message message, 
+                                              HttpURLConnection connection,
+                                              boolean needToCacheRequest, 
+                                              boolean isChunking,
+                                              int chunkThreshold) {
+        return new WrappedOutputStream(message, connection,
+                                       needToCacheRequest, 
+                                       isChunking,
+                                       chunkThreshold,
+                                       getConduitName());
+    }
+
     private HttpAuthSupplier createAuthSupplier(String authType) {
         if (HttpAuthHeader.AUTH_TYPE_NEGOTIATE.equals(authType)) {
             return new SpnegoAuthSupplier();
@@ -671,7 +706,10 @@ public class HTTPConduit
             }
             //defaultEndpointURL = null;
         }
-    
+        
+        if (clientSidePolicy != null) {
+            clientSidePolicy.removePropertyChangeListener(this);
+        }
     }
 
     /**
@@ -820,8 +858,12 @@ public class HTTPConduit
      * method will override any HTTPClientPolicy set in configuration.
      */
     public void setClient(HTTPClientPolicy client) {
+        if (this.clientSidePolicy != null) {
+            this.clientSidePolicy.removePropertyChangeListener(this);
+        }
         this.clientSidePolicy = client;
-        client.addPropertyChangeListener(this);
+        clientSidePolicy.removePropertyChangeListener(this); //make sure we aren't added twice
+        clientSidePolicy.addPropertyChangeListener(this);
         endpointInfo.setProperty("org.apache.cxf.ws.addressing.replyto", client.getDecoupledEndpoint());
     }
 
@@ -947,6 +989,7 @@ public class HTTPConduit
         switch(responseCode) {
         case HttpURLConnection.HTTP_MOVED_PERM:
         case HttpURLConnection.HTTP_MOVED_TEMP:
+        case 307:
             connection = redirectRetransmit(origConnection, message, cachedStream);
             break;
         case HttpURLConnection.HTTP_UNAUTHORIZED:
@@ -1469,19 +1512,25 @@ public class HTTPConduit
                         }
                     }
                 };
-                Executor ex = outMessage.getExchange().get(Executor.class);
-                if (ex == null) {
-                    WorkQueueManager mgr = outMessage.getExchange().get(Bus.class)
-                        .getExtension(WorkQueueManager.class);
-                    ex = mgr.getNamedWorkQueue("http-conduit");
+                try {
+                    Executor ex = outMessage.getExchange().get(Executor.class);
                     if (ex == null) {
-                        ex = mgr.getAutomaticWorkQueue();
-                    }
-                } else {
-                    outMessage.getExchange().put(Executor.class.getName() 
+                        WorkQueueManager mgr = outMessage.getExchange().get(Bus.class)
+                            .getExtension(WorkQueueManager.class);
+                        AutomaticWorkQueue qu = mgr.getNamedWorkQueue("http-conduit");
+                        if (qu == null) {
+                            qu = mgr.getAutomaticWorkQueue();
+                        }
+                        qu.execute(runnable, 5000);
+                    } else {
+                        outMessage.getExchange().put(Executor.class.getName() 
                                                  + ".USING_SPECIFIED", Boolean.TRUE);
+                        ex.execute(runnable);
+                    }
+                } catch (RejectedExecutionException rex) {
+                    LOG.warning("EXECUTOR_FULL");
+                    handleResponseInternal();
                 }
-                ex.execute(runnable);
             }
         }
 
@@ -1498,6 +1547,7 @@ public class HTTPConduit
         protected void handleResponseInternal() throws IOException {
             Exchange exchange = outMessage.getExchange();
             int responseCode = connection.getResponseCode();
+            
             if (outMessage != null && exchange != null) {
                 exchange.put(Message.RESPONSE_CODE, responseCode);
             }
@@ -1514,12 +1564,23 @@ public class HTTPConduit
             }
 
             InputStream in = null;
-            if (isOneway(exchange)) {
+            // oneway or decoupled twoway calls may expect HTTP 202 with no content
+            if (isOneway(exchange) 
+                || HttpURLConnection.HTTP_ACCEPTED == responseCode) {
                 in = ChunkedUtil.getPartialResponse(connection, responseCode);
                 if (in == null) {
                     // oneway operation or decoupled MEP without 
                     // partial response
                     connection.getInputStream().close();
+                    ClientCallback cc = exchange.get(ClientCallback.class);
+                    if (null != cc) {
+                        //REVISIT move the decoupled destination property name into api
+                        Endpoint ep = exchange.getEndpoint();
+                        if (null != ep && null != ep.getEndpointInfo() && null == ep.getEndpointInfo().
+                            getProperty("org.apache.cxf.ws.addressing.MAPAggregator.decoupledDestination")) {
+                            cc.handleResponse(null, null);
+                        }
+                    }
                     return;
                 }
             } else {
@@ -1659,6 +1720,7 @@ public class HTTPConduit
         switch (e.responseCode()) {
         case HttpURLConnection.HTTP_MOVED_PERM: // 301
         case HttpURLConnection.HTTP_MOVED_TEMP: // 302
+        case 307:    
             msg += " that returned location header '" + e.getLocation() + "'";
             break;
         case HttpURLConnection.HTTP_UNAUTHORIZED: // 401
