@@ -46,18 +46,21 @@ import org.apache.cxf.service.model.EndpointInfo;
 import org.apache.cxf.transport.MessageObserver;
 import org.apache.cxf.transport.https.HttpsURLConnectionFactory;
 import org.apache.cxf.ws.addressing.EndpointReferenceType;
-import org.easymock.classextension.EasyMock;
-import org.easymock.classextension.IMocksControl;
+import org.easymock.EasyMock;
+import org.easymock.IMocksControl;
 
 import org.junit.After;
+import org.junit.AfterClass;
 import org.junit.Assert;
 import org.junit.Before;
+import org.junit.BeforeClass;
 import org.junit.Test;
 
 /**
  */
 public class HTTPConduitURLEasyMockTest extends Assert {
-    
+    private static String oldHttpProxyHost;
+
     private enum ResponseStyle { NONE, BACK_CHANNEL, BACK_CHANNEL_ERROR, DECOUPLED };
     private enum ResponseDelimiter { LENGTH, CHUNKED, EOF };
 
@@ -73,6 +76,22 @@ public class HTTPConduitURLEasyMockTest extends Assert {
     private OutputStream os;
     private InputStream is;
     
+    @BeforeClass
+    public static void disableHttpProxy() throws Exception {
+        oldHttpProxyHost = System.getProperty("http.proxyHost");
+        if (oldHttpProxyHost != null) {
+            // disable http proxy so that the connection mocking works (see setUpConduit)
+            System.clearProperty("http.proxyHost");
+        }
+    }
+    
+    @AfterClass
+    public static void revertHttpProxy() throws Exception {
+        if (oldHttpProxyHost != null) {
+            System.setProperty("http.proxyHost", oldHttpProxyHost);
+        }
+    }
+
     /**
      * This is an extension to the HTTPConduit that replaces
      * the dynamic assignment of the HttpURLConnectionFactory,
@@ -208,6 +227,22 @@ public class HTTPConduitURLEasyMockTest extends Assert {
         finalVerify();
     }
     
+    @Test
+    public void testSendTwowayDecoupledEmptyPartialResponse() 
+        throws Exception {
+        control = EasyMock.createNiceControl();
+        HTTPConduit conduit = setUpConduit(true, false);
+        Message message = new MessageImpl();
+        conduit.prepare(message);
+        verifySentMessage(conduit, 
+                          message, 
+                          ResponseStyle.DECOUPLED,
+                          ResponseDelimiter.EOF,
+                          true,  // empty response
+                          "POST");
+        finalVerify();
+    }
+
     private void setUpHeaders(Message message) {
         Map<String, List<String>> headers = new TreeMap<String, List<String>>(String.CASE_INSENSITIVE_ORDER);
         List<String> contentTypes = new ArrayList<String>();
@@ -234,6 +269,8 @@ public class HTTPConduitURLEasyMockTest extends Assert {
         EasyMock.expectLastCall().andReturn(true);
         exchange.isSynchronous();
         EasyMock.expectLastCall().andReturn(true);
+        exchange.isEmpty();
+        EasyMock.expectLastCall().andReturn(true).anyTimes();
     }
     
     private HTTPConduit setUpConduit(boolean send, boolean autoRedirect) throws Exception {
