@@ -21,8 +21,8 @@ package org.apache.cxf.ws.rm.soap;
 
 import java.io.ByteArrayInputStream;
 import java.io.IOException;
+import java.io.InputStream;
 import java.io.OutputStream;
-import java.math.BigInteger;
 import java.net.ConnectException;
 import java.util.ArrayList;
 import java.util.Arrays;
@@ -49,7 +49,6 @@ import org.apache.cxf.message.MessageUtils;
 import org.apache.cxf.service.model.EndpointInfo;
 import org.apache.cxf.transport.Conduit;
 import org.apache.cxf.transport.MessageObserver;
-import org.apache.cxf.workqueue.SynchronousExecutor;
 import org.apache.cxf.ws.addressing.AddressingProperties;
 import org.apache.cxf.ws.addressing.AttributedURIType;
 import org.apache.cxf.ws.policy.AssertionInfo;
@@ -117,7 +116,7 @@ public class RetransmissionQueueImpl implements RetransmissionQueue {
      * @param seq the sequence object.
      */
     public void purgeAcknowledged(SourceSequence seq) {
-        Collection<BigInteger> purged = new ArrayList<BigInteger>();
+        Collection<Long> purged = new ArrayList<Long>();
         synchronized (this) {
             LOG.fine("Start purging resend candidates.");
             List<ResendCandidate> sequenceCandidates = getSequenceCandidates(seq);
@@ -127,7 +126,7 @@ public class RetransmissionQueueImpl implements RetransmissionQueue {
                     RMProperties properties = RMContextUtils.retrieveRMProperties(candidate.getMessage(),
                                                                                   true);
                     SequenceType st = properties.getSequence();
-                    BigInteger m = st.getMessageNumber();
+                    long m = st.getMessageNumber().longValue();
                     if (seq.isAcknowledged(m)) {
                         sequenceCandidates.remove(i);
                         candidate.resolved();
@@ -338,19 +337,26 @@ public class RetransmissionQueueImpl implements RetransmissionQueue {
                     }
                 }
             }
-            byte[] content = (byte[])message
+            CachedOutputStream content = (CachedOutputStream)message
                 .get(RMMessageConstants.SAVED_CONTENT);
-            if (null == content) {                
-                content = message.getContent(byte[].class); 
+            InputStream bis = null;
+            if (null == content) {
+                byte[] savedbytes = message.getContent(byte[].class);
+                bis = new ByteArrayInputStream(savedbytes); 
                 if (LOG.isLoggable(Level.FINE)) {
-                    LOG.fine("Using saved byte array: " + Arrays.toString(content));
+                    LOG.fine("Using saved byte array: " + Arrays.toString(savedbytes));
                 }
             } else {
+                bis = content.getInputStream();
                 if (LOG.isLoggable(Level.FINE)) {
-                    LOG.fine("Using saved output stream: " + IOUtils.newStringFromBytes(content));
+                    if (content.size() < 65536) {
+                        LOG.fine("Using saved output stream: " 
+                                 + IOUtils.newStringFromBytes(content.getBytes()));                        
+                    } else {                        
+                        LOG.fine("Using saved output stream: ...");                        
+                    }
                 }
             }
-            ByteArrayInputStream bis = new ByteArrayInputStream(content);
 
             // copy saved output stream to new output stream in chunks of 1024
             IOUtils.copyAndCloseInput(bis, os);
@@ -421,11 +427,7 @@ public class RetransmissionQueueImpl implements RetransmissionQueue {
             Executor executor = ep.getExecutor();
             if (null == executor) {
                 executor = ep.getService().getExecutor();
-                if (executor == null) {
-                    executor = SynchronousExecutor.getInstance();
-                } else {
-                    LOG.log(Level.FINE, "Using service executor {0}", executor.getClass().getName());
-                }
+                LOG.log(Level.FINE, "Using service executor {0}", executor.getClass().getName());
             } else {
                 LOG.log(Level.FINE, "Using endpoint executor {0}", executor.getClass().getName());
             }
@@ -480,6 +482,7 @@ public class RetransmissionQueueImpl implements RetransmissionQueue {
             next = null;
             if (null != nextTask) {
                 nextTask.cancel();
+                releaseSavedMessage();
             }
         }
         
@@ -489,9 +492,17 @@ public class RetransmissionQueueImpl implements RetransmissionQueue {
         protected void cancel() {
             if (null != nextTask) {
                 nextTask.cancel();
+                releaseSavedMessage();
             }
         }
 
+        private void releaseSavedMessage() {
+            CachedOutputStream saved = (CachedOutputStream)message.get(RMMessageConstants.SAVED_CONTENT);
+            if (saved != null) {
+                saved.releaseTempFileHold();
+            }
+
+        }
         /**
          * @return associated message context
          */
