@@ -19,12 +19,14 @@
 
 package org.apache.cxf.ws.rm.persistence.jdbc;
 
+import java.io.ByteArrayInputStream;
 import java.io.IOException;
-import java.math.BigInteger;
+import java.io.InputStream;
 import java.sql.SQLException;
 import java.util.ArrayList;
 import java.util.Collection;
 import java.util.Date;
+import java.util.List;
 
 import org.apache.cxf.helpers.IOUtils;
 import org.apache.cxf.ws.addressing.v200408.EndpointReferenceType;
@@ -36,8 +38,8 @@ import org.apache.cxf.ws.rm.SequenceAcknowledgement;
 import org.apache.cxf.ws.rm.SourceSequence;
 import org.apache.cxf.ws.rm.persistence.RMMessage;
 import org.apache.cxf.ws.rm.persistence.RMStoreException;
-import org.easymock.classextension.EasyMock;
-import org.easymock.classextension.IMocksControl;
+import org.easymock.EasyMock;
+import org.easymock.IMocksControl;
 import org.junit.AfterClass;
 import org.junit.Assert;
 import org.junit.Before;
@@ -56,6 +58,10 @@ public class RMTxStoreTest extends Assert {
     private static final String NON_ANON_ACKS_TO = 
         "http://localhost:9999/decoupled_endpoint";
     
+    private static final Long ZERO = new Long(0);
+    private static final Long ONE = new Long(1);
+    private static final Long TEN = new Long(10);
+    
     private static RMTxStore store;    
     private static SequenceAcknowledgement ack1;
     private static SequenceAcknowledgement ack2;
@@ -74,18 +80,19 @@ public class RMTxStoreTest extends Assert {
         ack1 = new SequenceAcknowledgement();
         SequenceAcknowledgement.AcknowledgementRange range = 
             new SequenceAcknowledgement.AcknowledgementRange();
-        range.setLower(BigInteger.ONE);
-        range.setUpper(BigInteger.ONE);
+        
+        range.setLower(ONE);
+        range.setUpper(ONE);
         ack1.getAcknowledgementRange().add(range);
         
         ack2 = new SequenceAcknowledgement();
         range = new SequenceAcknowledgement.AcknowledgementRange();
-        range.setLower(BigInteger.ONE);
-        range.setUpper(BigInteger.ONE);
+        range.setLower(ONE);
+        range.setUpper(ONE);
         ack2.getAcknowledgementRange().add(range);
         range = new SequenceAcknowledgement.AcknowledgementRange();
-        range.setLower(new BigInteger("3"));
-        range.setUpper(BigInteger.TEN);
+        range.setLower(new Long(3));
+        range.setUpper(TEN);
         ack2.getAcknowledgementRange().add(range);
     }
     
@@ -221,9 +228,10 @@ public class RMTxStoreTest extends Assert {
         RMMessage msg = control.createMock(RMMessage.class);
         Identifier sid1 = RMUtils.getWSRMFactory().createIdentifier();
         sid1.setValue("sequence1");
-        EasyMock.expect(msg.getMessageNumber()).andReturn(BigInteger.ONE).times(2); 
+        EasyMock.expect(msg.getMessageNumber()).andReturn(ONE).times(2); 
         byte[] bytes = new byte[89];
-        EasyMock.expect(msg.getContent()).andReturn(bytes).times(2);
+        EasyMock.expect(msg.getInputStream()).andReturn(new ByteArrayInputStream(bytes));
+        EasyMock.expect(msg.getSize()).andReturn(bytes.length);
         
         control.replay();
         store.beginTransaction();
@@ -233,8 +241,9 @@ public class RMTxStoreTest extends Assert {
         control.verify();
         
         control.reset();
-        EasyMock.expect(msg.getMessageNumber()).andReturn(BigInteger.ONE); 
-        EasyMock.expect(msg.getContent()).andReturn(bytes);
+        EasyMock.expect(msg.getMessageNumber()).andReturn(ONE); 
+        EasyMock.expect(msg.getInputStream()).andReturn(new ByteArrayInputStream(bytes));
+        EasyMock.expect(msg.getSize()).andReturn(bytes.length);
         
         control.replay();
         store.beginTransaction();
@@ -247,8 +256,9 @@ public class RMTxStoreTest extends Assert {
         control.verify();
         
         control.reset();
-        EasyMock.expect(msg.getMessageNumber()).andReturn(BigInteger.TEN).times(2); 
-        EasyMock.expect(msg.getContent()).andReturn(bytes).times(2); 
+        EasyMock.expect(msg.getMessageNumber()).andReturn(TEN).times(2); 
+        EasyMock.expect(msg.getInputStream()).andReturn(new ByteArrayInputStream(bytes)); 
+        EasyMock.expect(msg.getSize()).andReturn(bytes.length);
         
         control.replay();
         store.beginTransaction();
@@ -257,11 +267,11 @@ public class RMTxStoreTest extends Assert {
         store.commit();
         control.verify();
         
-        Collection<BigInteger> messageNrs = new ArrayList<BigInteger>();
-        messageNrs.add(BigInteger.ZERO);
-        messageNrs.add(BigInteger.TEN);
-        messageNrs.add(BigInteger.ONE);
-        messageNrs.add(BigInteger.TEN);
+        Collection<Long> messageNrs = new ArrayList<Long>();
+        messageNrs.add(ZERO);
+        messageNrs.add(TEN);
+        messageNrs.add(ONE);
+        messageNrs.add(TEN);
         
         store.removeMessages(sid1, messageNrs, true);
         store.removeMessages(sid1, messageNrs, false);
@@ -286,7 +296,7 @@ public class RMTxStoreTest extends Assert {
         control.verify();
         
         control.reset();
-        EasyMock.expect(seq.getLastMessageNumber()).andReturn(null);
+        EasyMock.expect(seq.getLastMessageNumber()).andReturn(Long.valueOf(0));
         EasyMock.expect(seq.getAcknowledgment()).andReturn(ack1);        
         EasyMock.expect(seq.getIdentifier()).andReturn(sid1);
         
@@ -296,7 +306,7 @@ public class RMTxStoreTest extends Assert {
         store.abort();
         
         control.reset();
-        EasyMock.expect(seq.getLastMessageNumber()).andReturn(BigInteger.TEN);
+        EasyMock.expect(seq.getLastMessageNumber()).andReturn(TEN);
         EasyMock.expect(seq.getAcknowledgment()).andReturn(ack1);        
         EasyMock.expect(seq.getIdentifier()).andReturn(sid1);
         
@@ -323,7 +333,7 @@ public class RMTxStoreTest extends Assert {
         control.verify();        
         
         control.reset();
-        EasyMock.expect(seq.getCurrentMessageNr()).andReturn(BigInteger.ONE);
+        EasyMock.expect(seq.getCurrentMessageNr()).andReturn(ONE);
         EasyMock.expect(seq.isLastMessage()).andReturn(false);
         EasyMock.expect(seq.getIdentifier()).andReturn(sid1);   
         
@@ -333,7 +343,7 @@ public class RMTxStoreTest extends Assert {
         store.abort();
         
         control.reset();
-        EasyMock.expect(seq.getCurrentMessageNr()).andReturn(BigInteger.TEN);
+        EasyMock.expect(seq.getCurrentMessageNr()).andReturn(TEN);
         EasyMock.expect(seq.isLastMessage()).andReturn(true);  
         EasyMock.expect(seq.getIdentifier()).andReturn(sid1);
         
@@ -406,6 +416,66 @@ public class RMTxStoreTest extends Assert {
         }
     }
 
+    @Test
+    public void testGetDestinationSequence() throws SQLException, IOException {
+        
+        Identifier sid1 = null;
+        Identifier sid2 = null;
+        
+        DestinationSequence seq = store.getDestinationSequence(RMUtils.getWSRMFactory().createIdentifier());
+        assertNull(seq);
+
+        try {
+            sid1 = setupDestinationSequence("sequence1");
+
+            seq = store.getDestinationSequence(sid1);
+            assertNotNull(seq);
+            verifyDestinationSequence("sequence1", seq);
+
+            sid2 = setupDestinationSequence("sequence2");
+            seq = store.getDestinationSequence(sid2);
+            assertNotNull(seq);
+            verifyDestinationSequence("sequence2", seq);
+        } finally {
+            if (null != sid1) {
+                store.removeDestinationSequence(sid1);
+            }
+            if (null != sid2) {
+                store.removeDestinationSequence(sid2);
+            }
+        }
+    }
+
+    @Test
+    public void testGetSourceSequence() throws SQLException, IOException {
+        
+        Identifier sid1 = null;
+        Identifier sid2 = null;
+        
+        SourceSequence seq = store.getSourceSequence(RMUtils.getWSRMFactory().createIdentifier());
+        assertNull(seq);
+        
+        try {
+            sid1 = setupSourceSequence("sequence1");
+
+            seq = store.getSourceSequence(sid1);
+            assertNotNull(seq);
+            verifySourceSequence("sequence1", seq);
+            
+            sid2 = setupSourceSequence("sequence2");
+            seq = store.getSourceSequence(sid2);
+            assertNotNull(seq);
+            verifySourceSequence("sequence2", seq);
+        } finally {
+            if (null != sid1) {
+                store.removeSourceSequence(sid1);
+            }
+            if (null != sid2) {
+                store.removeSourceSequence(sid2);
+            }
+        }
+    }
+
     @Test
     public void testGetMessages() throws SQLException, IOException {
         
@@ -420,8 +490,8 @@ public class RMTxStoreTest extends Assert {
         assertEquals(0, out.size());
         
         try {
-            setupMessage(sid1, BigInteger.ONE, null, true);
-            setupMessage(sid1, BigInteger.ONE, null, false);
+            setupMessage(sid1, ONE, null, true);
+            setupMessage(sid1, ONE, null, false);
 
             out = store.getMessages(sid1, true);
             assertEquals(1, out.size());
@@ -431,8 +501,8 @@ public class RMTxStoreTest extends Assert {
             assertEquals(1, in.size());
             checkRecoveredMessages(in);
             
-            setupMessage(sid1, BigInteger.TEN, NON_ANON_ACKS_TO, true);
-            setupMessage(sid1, BigInteger.TEN, NON_ANON_ACKS_TO, false);
+            setupMessage(sid1, TEN, NON_ANON_ACKS_TO, true);
+            setupMessage(sid1, TEN, NON_ANON_ACKS_TO, false);
             
             out = store.getMessages(sid1, true);
             assertEquals(2, out.size());
@@ -442,9 +512,9 @@ public class RMTxStoreTest extends Assert {
             assertEquals(2, in.size());
             checkRecoveredMessages(in);
         } finally {
-            Collection<BigInteger> msgNrs = new ArrayList<BigInteger>();
-            msgNrs.add(BigInteger.ONE);
-            msgNrs.add(BigInteger.TEN);
+            Collection<Long> msgNrs = new ArrayList<Long>();
+            msgNrs.add(ONE);
+            msgNrs.add(TEN);
          
             store.removeMessages(sid1, msgNrs, true);
             store.removeMessages(sid1, msgNrs, false);
@@ -459,11 +529,11 @@ public class RMTxStoreTest extends Assert {
         EndpointReferenceType epr = RMUtils.createAnonymousReference2004();
         
         SequenceAcknowledgement ack = ack1;
-        BigInteger lmn = null;
+        Long lmn = ZERO;
          
         if ("sequence2".equals(s)) {
             ack = ack2;
-            lmn = BigInteger.TEN;
+            lmn = TEN;
         }
         
         EasyMock.expect(seq.getIdentifier()).andReturn(sid);
@@ -490,14 +560,14 @@ public class RMTxStoreTest extends Assert {
             
         Date expiry = null;
         Identifier osid = null;
-        BigInteger cmn = BigInteger.ONE;
+        Long cmn = ONE;
         boolean lm = false;
         
         if ("sequence2".equals(s)) {
             expiry = new Date(System.currentTimeMillis() + 3600 * 1000);
             osid = RMUtils.getWSRMFactory().createIdentifier();
             osid.setValue("offeringSequence");
-            cmn = BigInteger.TEN;
+            cmn = TEN;
             lm = true;            
         } 
         
@@ -518,7 +588,7 @@ public class RMTxStoreTest extends Assert {
         
         return sid;
     }
-    /*
+
     private void verifyDestinationSequence(String s, DestinationSequence seq) {
         Identifier sid = seq.getIdentifier();
         assertNotNull(sid);
@@ -552,7 +622,7 @@ public class RMTxStoreTest extends Assert {
             assertTrue(seq.isLastMessage());
         }
     }
-
+    
     private void verifyAcknowledgementRanges(List<SequenceAcknowledgement.AcknowledgementRange> ranges, 
                                              long[] values) {
         assertNotNull(ranges);
@@ -560,18 +630,19 @@ public class RMTxStoreTest extends Assert {
         
         int v = 0;
         for (SequenceAcknowledgement.AcknowledgementRange range : ranges) {
-            assertEquals(values[v++], (long)range.getLower().longValue());   
-            assertEquals(values[v++], (long)range.getUpper().longValue());   
+            assertEquals(values[v++], (long)range.getLower());   
+            assertEquals(values[v++], (long)range.getUpper());   
         }
     }
-    */    
-    private void setupMessage(Identifier sid, BigInteger mn, String to, boolean outbound) 
+    
+    private void setupMessage(Identifier sid, Long mn, String to, boolean outbound) 
         throws IOException, SQLException  {
         RMMessage msg = control.createMock(RMMessage.class);
         EasyMock.expect(msg.getMessageNumber()).andReturn(mn);
         EasyMock.expect(msg.getTo()).andReturn(to);
-        String value = "Message " + mn.longValue();
-        EasyMock.expect(msg.getContent()).andReturn(value.getBytes());
+        byte[] value = ("Message " + mn.longValue()).getBytes();
+        EasyMock.expect(msg.getInputStream()).andReturn(new ByteArrayInputStream(value));
+        EasyMock.expect(msg.getSize()).andReturn(value.length);
         
         control.replay();
         store.beginTransaction();
@@ -587,22 +658,22 @@ public class RMTxStoreTest extends Assert {
                                           || "sequence2".equals(recovered.getIdentifier().getValue()));
             assertEquals(RMConstants.getAnonymousAddress(), recovered.getAcksTo().getAddress().getValue());
             if ("sequence1".equals(recovered.getIdentifier().getValue())) {                      
-                assertNull(recovered.getLastMessageNumber());                
+                assertEquals(0, recovered.getLastMessageNumber());                
                 assertEquals(1, recovered.getAcknowledgment().getAcknowledgementRange().size());
                 SequenceAcknowledgement.AcknowledgementRange r = 
                     recovered.getAcknowledgment().getAcknowledgementRange().get(0);
-                assertEquals(BigInteger.ONE, r.getLower());
-                assertEquals(BigInteger.ONE, r.getUpper());
+                assertEquals(ONE, r.getLower());
+                assertEquals(ONE, r.getUpper());
             } else {
-                assertEquals(BigInteger.TEN, recovered.getLastMessageNumber());
+                assertEquals(10, recovered.getLastMessageNumber());
                 assertEquals(2, recovered.getAcknowledgment().getAcknowledgementRange().size());
                 SequenceAcknowledgement.AcknowledgementRange r = 
                     recovered.getAcknowledgment().getAcknowledgementRange().get(0);
-                assertEquals(BigInteger.ONE, r.getLower());
-                assertEquals(BigInteger.ONE, r.getUpper());
+                assertEquals(ONE, r.getLower());
+                assertEquals(ONE, r.getUpper());
                 r = recovered.getAcknowledgment().getAcknowledgementRange().get(1);
-                assertEquals(new BigInteger("3"), r.getLower());
-                assertEquals(BigInteger.TEN, r.getUpper());                
+                assertEquals(new Long(3), r.getLower());
+                assertEquals(TEN, r.getUpper());                
             }
         }
     }
@@ -614,12 +685,12 @@ public class RMTxStoreTest extends Assert {
                                           || "sequence2".equals(recovered.getIdentifier().getValue()));
             if ("sequence1".equals(recovered.getIdentifier().getValue())) {                      
                 assertFalse(recovered.isLastMessage());
-                assertEquals(BigInteger.ONE, recovered.getCurrentMessageNr());  
+                assertEquals(1, recovered.getCurrentMessageNr());  
                 assertNull(recovered.getExpires());
                 assertNull(recovered.getOfferingSequenceIdentifier());
             } else {
                 assertTrue(recovered.isLastMessage());
-                assertEquals(BigInteger.TEN, recovered.getCurrentMessageNr()); 
+                assertEquals(10, recovered.getCurrentMessageNr()); 
                 assertNotNull(recovered.getExpires());
                 assertEquals("offeringSequence", recovered.getOfferingSequenceIdentifier().getValue());
             }
@@ -628,15 +699,19 @@ public class RMTxStoreTest extends Assert {
     
     private void checkRecoveredMessages(Collection<RMMessage> msgs) {
         for (RMMessage msg : msgs) {
-            BigInteger mn = msg.getMessageNumber();
-            assertTrue(BigInteger.ONE.equals(mn) || BigInteger.TEN.equals(mn));
-            if (BigInteger.TEN.equals(mn)) {
+            long mn = msg.getMessageNumber();
+            assertTrue(mn == 1 || mn == 10);
+            if (mn == 10) {
                 assertEquals(NON_ANON_ACKS_TO, msg.getTo());
             } else {
                 assertNull(msg.getTo());
             }
-            byte[] actual = msg.getContent();
-            assertEquals(new String("Message " + mn.longValue()), IOUtils.newStringFromBytes(actual));
+            try {
+                InputStream actual = msg.getInputStream();
+                assertEquals(new String("Message " + mn), IOUtils.readStringFromStream(actual));
+            } catch (IOException e) {
+                fail("failed to get the input stream");
+            }
         }
     }
     
