@@ -21,7 +21,6 @@ package org.apache.cxf.ws.rm.soap;
 
 import java.io.InputStream;
 import java.lang.reflect.Method;
-import java.math.BigInteger;
 import java.util.ArrayList;
 import java.util.Collection;
 import java.util.Iterator;
@@ -53,6 +52,7 @@ import org.apache.cxf.ws.rm.RMContextUtils;
 import org.apache.cxf.ws.rm.RMProperties;
 import org.apache.cxf.ws.rm.RMUtils;
 import org.apache.cxf.ws.rm.SequenceAcknowledgement;
+import org.apache.cxf.ws.rm.SequenceAcknowledgement.AcknowledgementRange;
 import org.apache.cxf.ws.rm.SequenceFault;
 import org.apache.cxf.ws.rm.SequenceFaultType;
 import org.apache.cxf.ws.rm.SequenceType;
@@ -67,8 +67,10 @@ import org.junit.Test;
 public class RMSoapInterceptorTest extends Assert {
 
     private static final String SEQ_IDENTIFIER = "http://Business456.com/RM/ABC";
-    private static final BigInteger MSG1_MESSAGE_NUMBER = BigInteger.ONE;
-    private static final BigInteger MSG2_MESSAGE_NUMBER = BigInteger.ONE.add(BigInteger.ONE);
+    private static final Long ONE = new Long(1);
+    private static final Long TEN = new Long(10);
+    private static final Long MSG1_MESSAGE_NUMBER = ONE;
+    private static final Long MSG2_MESSAGE_NUMBER = new Long(2);
 
     private IMocksControl control;
     
@@ -274,10 +276,44 @@ public class RMSoapInterceptorTest extends Assert {
         assertNotNull(ack);
         assertEquals(ack.getIdentifier().getValue(), SEQ_IDENTIFIER);
         assertEquals(2, ack.getAcknowledgementRange().size());
+        AcknowledgementRange r1 = ack.getAcknowledgementRange().get(0);
+        AcknowledgementRange r2 = ack.getAcknowledgementRange().get(1);
+        verifyRange(r1, 1, 1);
+        verifyRange(r2, 3, 3);
         assertNull(rmps.getSequence());
         assertNull(rmps.getAcksRequested());
     }
 
+    @Test
+    public void testDecodeAcknowledgements2() throws XMLStreamException {
+        SoapMessage message = setUpInboundMessage("resources/Acknowledgment2.xml");
+        RMSoapInterceptor codec = new RMSoapInterceptor();
+        codec.handleMessage(message);
+        RMProperties rmps = RMContextUtils.retrieveRMProperties(message, false);
+        Collection<SequenceAcknowledgement> acks = rmps.getAcks();
+        assertNotNull(acks);
+        assertEquals(1, acks.size());
+        SequenceAcknowledgement ack = acks.iterator().next();
+        assertNotNull(ack);
+        assertEquals(1, ack.getAcknowledgementRange().size());
+        AcknowledgementRange r1 = ack.getAcknowledgementRange().get(0);
+        verifyRange(r1, 1, 3);
+        assertNull(rmps.getSequence());
+        assertNull(rmps.getAcksRequested());
+    }
+
+    private void verifyRange(AcknowledgementRange r, int i, int j) {
+        assertNotNull(r);
+        if (i > 0) {
+            assertNotNull(r.getLower());
+            assertEquals(i, r.getLower().longValue());
+        }
+        if (j > 0) {
+            assertNotNull(r.getUpper());
+            assertEquals(j, r.getUpper().longValue());
+        }
+    }
+
     @Test
     public void testDecodeAcksRequested() throws XMLStreamException {
         SoapMessage message = setUpInboundMessage("resources/Retransmission.xml");
@@ -305,25 +341,25 @@ public class RMSoapInterceptorTest extends Assert {
         Identifier sid = factory.createIdentifier();
         sid.setValue("sequence1");
         s1.setIdentifier(sid);
-        s1.setMessageNumber(BigInteger.ONE);
+        s1.setMessageNumber(ONE);
         s2 = factory.createSequenceType();
         sid = factory.createIdentifier();
         sid.setValue("sequence2");
         s2.setIdentifier(sid);
-        s2.setMessageNumber(BigInteger.TEN);
+        s2.setMessageNumber(TEN);
 
         ack1 = factory.createSequenceAcknowledgement();
         SequenceAcknowledgement.AcknowledgementRange r = 
             factory.createSequenceAcknowledgementAcknowledgementRange();
-        r.setLower(BigInteger.ONE);
-        r.setUpper(BigInteger.ONE);
+        r.setLower(ONE);
+        r.setUpper(ONE);
         ack1.getAcknowledgementRange().add(r);
         ack1.setIdentifier(s1.getIdentifier());
 
         ack2 = factory.createSequenceAcknowledgement();
         r = factory.createSequenceAcknowledgementAcknowledgementRange();
-        r.setLower(BigInteger.ONE);
-        r.setUpper(BigInteger.TEN);
+        r.setLower(ONE);
+        r.setUpper(TEN);
         ack2.getAcknowledgementRange().add(r);
         ack2.setIdentifier(s2.getIdentifier());
 
