@@ -65,6 +65,7 @@ import javax.ws.rs.core.Response;
 import javax.ws.rs.core.SecurityContext;
 import javax.ws.rs.core.StreamingOutput;
 import javax.ws.rs.core.UriBuilder;
+import javax.ws.rs.core.UriInfo;
 import javax.xml.bind.JAXBElement;
 import javax.xml.bind.annotation.adapters.XmlAdapter;
 import javax.xml.bind.annotation.adapters.XmlJavaTypeAdapter;
@@ -74,9 +75,12 @@ import javax.xml.transform.dom.DOMSource;
 import org.apache.cxf.annotations.GZIP;
 import org.apache.cxf.common.util.ProxyHelper;
 import org.apache.cxf.helpers.XMLUtils;
+import org.apache.cxf.jaxrs.ext.Nullable;
 import org.apache.cxf.jaxrs.ext.Oneway;
 import org.apache.cxf.jaxrs.ext.search.SearchCondition;
 import org.apache.cxf.jaxrs.ext.search.SearchContext;
+import org.apache.cxf.jaxrs.ext.xml.XMLInstruction;
+import org.apache.cxf.jaxrs.ext.xml.XSISchemaLocation;
 import org.apache.cxf.phase.PhaseInterceptorChain;
 
 @Path("/bookstore")
@@ -88,6 +92,9 @@ public class BookStore {
     private long bookId = 123;
     private long cdId = 123;
     
+    private String defaultName;
+    private long defaultId;
+    
     private String currentBookId;
     @PathParam("CDId")
     private String currentCdId;
@@ -95,6 +102,8 @@ public class BookStore {
     private HttpHeaders httpHeaders;
     @Context 
     private SecurityContext securityContext;
+    @Context 
+    private UriInfo ui;
     
     public BookStore() {
         init();
@@ -102,12 +111,25 @@ public class BookStore {
     
     @PostConstruct
     public void postConstruct() {
-        System.out.println("PostConstruct called");
+        //System.out.println("PostConstruct called");
     }
     
     @PreDestroy
     public void preDestroy() {
-        System.out.println("PreDestroy called");
+        //System.out.println("PreDestroy called");
+    }
+
+    @GET
+    @Path("allCharsButA-B/:@!$&'()*+,;=-._~")
+    public Book getWithComplexPath() {
+        return new Book("Encoded Path", 125L);
+    }
+    
+    @GET
+    @Path("/default")
+    @Produces("application/xml")
+    public Book getDefaultBook() {
+        return new Book(defaultName, defaultId);
     }
     
     @GET
@@ -122,7 +144,21 @@ public class BookStore {
     @POST
     @Path("emptypost")
     public void emptypost() {
-        System.out.println("empty post");
+        String uri = ui.getAbsolutePath().toString();
+        //System.out.println(uri);
+        if (uri.endsWith("/")) {
+            throw new WebApplicationException(400);
+        }
+    }
+    
+    @PUT
+    @Path("emptyput")
+    public void emptyput() {
+    }
+    
+    @POST
+    public void emptypostNoPath() {
+        emptypost();
     }
     
     @GET
@@ -181,6 +217,8 @@ public class BookStore {
     @GET
     @Path("name-in-query")
     @Produces("application/xml")
+    @XMLInstruction("<!DOCTYPE Something SYSTEM 'my.dtd'><?xmlstylesheet href='common.css'?>")
+    @XSISchemaLocation("book.xsd")
     public Book getBookFromQuery(@QueryParam("name") String name) {
         return new Book(name, 321L);
     }
@@ -276,6 +314,27 @@ public class BookStore {
         return bs;
     }
     
+    @POST
+    @Path("/collectionBook")
+    @Produces({"application/xml", "application/json" })
+    @Consumes({"application/xml", "application/json" })
+    public List<Book> postBookGetCollection(Book book) throws Exception {
+        List<Book> list = new ArrayList<Book>();
+        list.add(book);
+        return list;
+    }
+    
+    @POST
+    @Path("/collections3")
+    @Produces({"application/xml", "application/json" })
+    @Consumes({"application/xml", "application/json" })
+    public Book postCollectionGetBook(List<Book> bs) throws Exception {
+        if (bs == null || bs.size() != 2) {
+            throw new RuntimeException();
+        }
+        return bs.get(0);
+    }
+    
     @POST
     @Path("/collections2")
     @Produces({"application/xml", "application/json" })
@@ -334,6 +393,14 @@ public class BookStore {
         return doGetBook(s1 + s2);
     }
     
+    @GET
+    @Path("/segment/matrix-middle")
+    public Book getBookByMatrixParamsMiddle(@MatrixParam("first") String s1,
+                                      @MatrixParam("second") String s2) throws Exception {
+        
+        return doGetBook(s1 + s2);
+    }
+    
     @GET
     @Path("/segment/matrix-list")
     public Book getBookByMatrixListParams(@MatrixParam("first") List<String> list) throws Exception {
@@ -610,6 +677,11 @@ public class BookStore {
         currentBookId = id;
     }
     
+    public void setDefaultNameAndId(String name, long id) {
+        defaultName = name;
+        defaultId = id;
+    }
+    
     @GET
     @Path("/books/{bookId}/")
     @Produces("application/json;q=0.9")
@@ -681,6 +753,17 @@ public class BookStore {
         }
     }
 
+    @POST
+    @Path("/books/null")
+    @Produces("application/xml")
+    @Consumes("application/xml")
+    public Book handleNullBook(@Nullable Book book) {
+        if (book != null) {
+            throw new WebApplicationException(400);
+        }
+        return new Book("Default Book", 222L);
+    }
+    
     @POST
     @Path("/books")
     @Produces("text/xml")
@@ -929,7 +1012,14 @@ public class BookStore {
         return new BookSubresourceImpl();
     }
     
-    final void init() {
+    @GET
+    @Path("/reset")
+    public final String init() {
+        books.clear();
+        cds.clear();
+        bookId = 123;
+        cdId = 123;        
+        
         Book book = new Book();
         book.setId(bookId);
         book.setName("CXF in Action");
@@ -943,10 +1033,11 @@ public class BookStore {
         cd1.setId(++cdId);
         cd1.setName("BICYCLE RACE");
         cds.put(cd1.getId(), cd1);
+        return "OK";
     }
     
     @XmlJavaTypeAdapter(BookInfoAdapter2.class)
-    static interface BookInfoInterface {
+    interface BookInfoInterface {
         String getName();
         
         long getId();
