@@ -60,22 +60,46 @@ import org.apache.cxf.systest.jaxrs.BookStore.BookInfo;
 import org.apache.cxf.systest.jaxrs.BookStore.BookInfoInterface;
 import org.apache.cxf.testutil.common.AbstractBusClientServerTestBase;
 
+import org.junit.After;
 import org.junit.BeforeClass;
 import org.junit.Ignore;
 import org.junit.Test;
 
 public class JAXRSClientServerBookTest extends AbstractBusClientServerTestBase {
     public static final String PORT = BookServer.PORT;
+    public static final String PORT2 = allocatePort(JAXRSClientServerBookTest.class);
 
     @BeforeClass
     public static void startServers() throws Exception {
         assertTrue("server did not launch correctly",
                    launchServer(BookServer.class));
     }
+    @After
+    public void resetBookServer() throws Exception {
+        new URL("http://localhost:" + PORT + "/bookstore/reset").openStream().close();
+    }
+    
+    @Test
+    public void testWithComplexPath() {
+        WebClient wc = 
+            WebClient.create("http://localhost:" + PORT + "/bookstore/allCharsButA-B/:@!$&'()*+,;=-._~");
+        wc.accept("application/xml");
+        Book book = wc.get(Book.class);
+        assertEquals("Encoded Path", book.getName());
+    }
+    
+    @Test
+    public void testMalformedAcceptType() {
+        WebClient wc = 
+            WebClient.create("http://localhost:" + PORT + "/bookstore/books/123");
+        wc.accept("application");
+        Response r = wc.get();
+        assertEquals(406, r.getStatus());
+    }
     
     @Test
     public void testProxyWrongAddress() throws Exception {
-        BookStore store = JAXRSClientFactory.create("http://localhost:8080/wrongaddress", 
+        BookStore store = JAXRSClientFactory.create("http://localhost:" + PORT2 + "/wrongaddress", 
                                                     BookStore.class);
         try {
             store.getBook("123");
@@ -123,6 +147,20 @@ public class JAXRSClientServerBookTest extends AbstractBusClientServerTestBase {
         assertEquals(name, b.getName());
     }
     
+    @Test
+    public void testProcessingInstruction() throws Exception {
+        String base = "http://localhost:" + PORT;
+        String endpointAddress = base + "/bookstore/name-in-query"; 
+        WebClient wc = WebClient.create(endpointAddress);
+        String name = "Many        spaces";
+        wc.query("name", name);
+        String content = wc.get(String.class);
+        assertTrue(content.contains("<!DOCTYPE Something SYSTEM 'my.dtd'>"));
+        assertTrue(content.contains("<?xmlstylesheet href='" + base + "/common.css'?>"));
+        assertTrue(content.contains("xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\""));
+        assertTrue(content.contains("xsi:schemaLocation=\"" + base + "/book.xsd\""));
+    }
+    
     @Test
     public void testGetBookWithColonMarks() throws Exception {
         
@@ -167,6 +205,65 @@ public class JAXRSClientServerBookTest extends AbstractBusClientServerTestBase {
         assertEquals(123L, book.getId());
     }
     
+    @Test
+    public void testPostCollectionGetBooksWebClient() throws Exception {
+        
+        String endpointAddress =
+            "http://localhost:" + PORT + "/bookstore/collections3"; 
+        WebClient wc = WebClient.create(endpointAddress);
+        wc.accept("application/xml").type("application/xml");
+        Book b1 = new Book("CXF in Action", 123L);
+        Book b2 = new Book("CXF Rocks", 124L);
+        List<Book> books = new ArrayList<Book>();
+        books.add(b1);
+        books.add(b2);
+        Book book = wc.postCollection(books, Book.class, Book.class);
+        assertEquals(200, wc.getResponse().getStatus());
+        assertNotSame(b1, book);
+        assertEquals(b1.getName(), book.getName());
+    }
+    
+    @Test
+    public void testPostCollectionOfBooksWebClient() throws Exception {
+        
+        String endpointAddress =
+            "http://localhost:" + PORT + "/bookstore/collections"; 
+        WebClient wc = WebClient.create(endpointAddress);
+        wc.accept("application/xml").type("application/xml");
+        Book b1 = new Book("CXF in Action", 123L);
+        Book b2 = new Book("CXF Rocks", 124L);
+        List<Book> books = new ArrayList<Book>();
+        books.add(b1);
+        books.add(b2);
+        List<Book> books2 = new ArrayList<Book>(wc.postAndGetCollection(books, Book.class, Book.class));
+        assertNotNull(books2);
+        assertNotSame(books, books2);
+        assertEquals(2, books2.size());
+        Book b11 = books.get(0);
+        assertEquals(123L, b11.getId());
+        assertEquals("CXF in Action", b11.getName());
+        Book b22 = books.get(1);
+        assertEquals(124L, b22.getId());
+        assertEquals("CXF Rocks", b22.getName());
+        assertEquals(200, wc.getResponse().getStatus());
+    }
+    
+    @Test
+    public void testPostObjectGetCollection() throws Exception {
+        
+        String endpointAddress =
+            "http://localhost:" + PORT + "/bookstore/collectionBook"; 
+        WebClient wc = WebClient.create(endpointAddress);
+        wc.accept("application/xml").type("application/xml");
+        Book b1 = new Book("Book", 666L);
+        List<Book> books = new ArrayList<Book>(wc.postObjectGetCollection(b1, Book.class));
+        assertNotNull(books);
+        assertEquals(1, books.size());
+        Book b = books.get(0);
+        assertEquals(666L, b.getId());
+        assertEquals("Book", b.getName());
+    }
+    
     @Test
     public void testGetBookResponseAndETag() throws Exception {
         
@@ -302,6 +399,18 @@ public class JAXRSClientServerBookTest extends AbstractBusClientServerTestBase {
         }
     }
     
+    @Test
+    public void testServerWebApplicationExceptionResponse() throws Exception {
+        WebClient wc = WebClient.create("http://localhost:" + PORT + "/bookstore/webappexception");
+        wc.accept("application/xml");
+        try {
+            Response r = wc.get(Response.class);
+            assertEquals(500, r.getStatus());
+        } catch (ServerWebApplicationException ex) {
+            fail("Unexpected exception");
+        }
+    }
+    
     @Test
     public void testServerWebApplicationExceptionXML() throws Exception {
         ResponseReader reader = new ResponseReader();
@@ -538,18 +647,45 @@ public class JAXRSClientServerBookTest extends AbstractBusClientServerTestBase {
                              + PORT + "/bookstore/emptypost");
         Response response = wc.post(null);
         assertEquals(204, response.getStatus());
+        assertNull(response.getMetadata().getFirst("Content-Type"));
+    }
+    
+    @Test
+    public void testEmptyPut() throws Exception {
+        WebClient wc = 
+            WebClient.create("http://localhost:" 
+                             + PORT + "/bookstore/emptyput");
+        Response response = wc.put(null);
+        assertEquals(204, response.getStatus());
+        assertNull(response.getMetadata().getFirst("Content-Type"));
+        
+        response = wc.put("");
+        assertEquals(204, response.getStatus());
+        assertNull(response.getMetadata().getFirst("Content-Type"));
     }
     
     @Test
     public void testEmptyPostProxy() throws Exception {
+        String address = "http://localhost:" + PORT;
         JAXRSClientFactoryBean bean = new JAXRSClientFactoryBean(); 
-        bean.setAddress("http://localhost:" + PORT);
+        bean.setAddress(address);
         bean.setResourceClass(BookStore.class);
         BookStore store = bean.create(BookStore.class);
         store.emptypost();
         assertEquals(204, WebClient.client(store).getResponse().getStatus());
     }
     
+    @Test
+    public void testEmptyPostProxy2() throws Exception {
+        String address = "http://localhost:" + PORT;
+        JAXRSClientFactoryBean bean = new JAXRSClientFactoryBean();
+        bean.setAddress(address);
+        bean.setResourceClass(BookStore.class);
+        BookStore store = bean.create(BookStore.class);
+        store.emptypostNoPath();
+        assertEquals(204, WebClient.client(store).getResponse().getStatus());
+    }
+    
     @Test
     public void testGetBookByEncodedQuery() throws Exception {
         getAndCompareAsStrings("http://localhost:" + PORT + "/bookstore/bookquery?"
@@ -858,6 +994,47 @@ public class JAXRSClientServerBookTest extends AbstractBusClientServerTestBase {
                                "application/xml", 200);
     }
     
+    @Test
+    public void testGetBookSimple222() throws Exception {
+        WebClient wc = WebClient.create("http://localhost:" + PORT + "/simplebooks/222");
+        Book book = wc.get(Book.class);
+        assertEquals(222L, book.getId());
+    }
+    
+    @Test
+    public void testGetBookSimple() throws Exception {
+        WebClient wc = WebClient.create("http://localhost:" + PORT + "/simplebooks/simple");
+        Book book = wc.get(Book.class);
+        assertEquals(444L, book.getId());
+    }
+    
+    @Test
+    public void testFormattedJSON() {
+        WebClient wc = WebClient.create("http://localhost:" + PORT + "/bookstore/books/123");
+        wc.accept("application/json");
+        String response = wc.get(String.class);
+        // {"Book":{"id":123,"name":"CXF in Action"}}
+        
+        assertTrue(response.startsWith("{"));
+        assertTrue(response.endsWith("}"));
+        assertTrue(response.contains("\"Book\":{"));
+        assertTrue(response.indexOf("\"Book\":{") == 1);
+        
+        wc.query("_format", "");
+        response = wc.get(String.class);
+        //{
+        //    "Book":{
+        //      "id":123,
+        //      "name":"CXF in Action"
+        //    }
+        //}
+        assertTrue(response.startsWith("{"));
+        assertTrue(response.endsWith("}"));
+        assertTrue(response.contains("\"Book\":{"));
+        assertFalse(response.indexOf("\"Book\":{") == 1);
+        
+    }
+    
     @Test
     public void testGetBook123() throws Exception {
         getAndCompareAsStrings("http://localhost:" + PORT + "/bookstore/books/123",
@@ -932,6 +1109,14 @@ public class JAXRSClientServerBookTest extends AbstractBusClientServerTestBase {
                                "application/xml", 200);
     }
     
+    @Test
+    public void testGetBookByMatrixParametersInTheMiddle() throws Exception {
+        getAndCompareAsStrings(
+            "http://localhost:" + PORT + "/bookstore/segment;first=12;second=3/matrix-middle",
+            "resources/expected_get_book123.txt",
+            "application/xml", 200);
+    }
+    
     @Test
     public void testGetBookByHeader() throws Exception {
         getAndCompareAsStrings("http://localhost:" + PORT + "/bookstore/bookheaders",
@@ -1151,6 +1336,19 @@ public class JAXRSClientServerBookTest extends AbstractBusClientServerTestBase {
         }
     }
     
+    @Test
+    public void testAddBookEmptyContent() throws Exception {
+        Response r = WebClient.create("http://localhost:" + PORT + "/bookstore/books").post(null);
+        assertEquals(400, r.getStatus());
+    }
+    
+    @Test
+    public void testAddBookEmptyContentWithNullable() throws Exception {
+        Book defaultBook = 
+            WebClient.create("http://localhost:" + PORT + "/bookstore/books/null").post(null, Book.class);
+        assertEquals("Default Book", defaultBook.getName());
+    }
+    
     @Test
     public void testAddBook() throws Exception {
         doAddBook("http://localhost:" + PORT + "/bookstore/books");               
@@ -1477,9 +1675,9 @@ public class JAXRSClientServerBookTest extends AbstractBusClientServerTestBase {
     public void testUriInfoMatchedResources() throws Exception {
         getAndCompare("http://localhost:" + PORT + "/bookstore/"
                       + "booksubresource/123/chapters/sub/1/matched-resources", 
-                      "[class org.apache.cxf.systest.jaxrs.BookStore, " 
+                      "[class org.apache.cxf.systest.jaxrs.Chapter, " 
                       + "class org.apache.cxf.systest.jaxrs.Book, "
-                      + "class org.apache.cxf.systest.jaxrs.Chapter]", 
+                      + "class org.apache.cxf.systest.jaxrs.BookStore]", 
                       "text/plain", "text/plain", 200);
     }
     
@@ -1487,17 +1685,17 @@ public class JAXRSClientServerBookTest extends AbstractBusClientServerTestBase {
     public void testUriInfoMatchedResourcesWithObject() throws Exception {
         getAndCompare("http://localhost:" + PORT + "/bookstore/"
                       + "booksubresource/123/chaptersobject/sub/1/matched-resources", 
-                      "[class org.apache.cxf.systest.jaxrs.BookStore, " 
+                      "[class org.apache.cxf.systest.jaxrs.Chapter, " 
                       + "class org.apache.cxf.systest.jaxrs.Book, "
-                      + "class org.apache.cxf.systest.jaxrs.Chapter]", 
+                      + "class org.apache.cxf.systest.jaxrs.BookStore]", 
                       "text/plain", "text/plain", 200);
     }
 
     @Test
     public void testUriInfoMatchedUrisDecode() throws Exception {
-        String expected = "[/bookstore/booksubresource/123/, "
+        String expected = "[/bookstore/booksubresource/123/chapters/sub/1/matched!uris, "
                           + "/bookstore/booksubresource/123/chapters/sub/1/, "
-                          + "/bookstore/booksubresource/123/chapters/sub/1/matched!uris]";
+                          + "/bookstore/booksubresource/123/]";
         getAndCompare("http://localhost:" + PORT + "/bookstore/"
                       + "booksubresource/123/chapters/sub/1/matched%21uris?decode=true", 
                       expected, "text/plain", "text/plain", 200);
@@ -1506,9 +1704,9 @@ public class JAXRSClientServerBookTest extends AbstractBusClientServerTestBase {
     @Test
     public void testUriInfoMatchedUrisNoDecode() throws Exception {
         //note '%21' instead of '!'
-        String expected = "[/bookstore/booksubresource/123/, "
+        String expected = "[/bookstore/booksubresource/123/chapters/sub/1/matched%21uris, "
             + "/bookstore/booksubresource/123/chapters/sub/1/, "
-            + "/bookstore/booksubresource/123/chapters/sub/1/matched%21uris]";
+            + "/bookstore/booksubresource/123/]";
         getAndCompare("http://localhost:" + PORT + "/bookstore/"
                       + "booksubresource/123/chapters/sub/1/matched%21uris?decode=false", 
                       expected,
