@@ -28,21 +28,31 @@ import java.net.URL;
 import java.net.URLConnection;
 import java.util.Collections;
 import java.util.HashMap;
+import java.util.List;
 import java.util.Map;
 
+import javax.ws.rs.core.Response;
 import javax.xml.bind.annotation.XmlElement;
 import javax.xml.bind.annotation.XmlRootElement;
 
+import org.w3c.dom.Document;
+import org.w3c.dom.Element;
+
 import org.apache.commons.httpclient.HttpClient;
 import org.apache.commons.httpclient.methods.FileRequestEntity;
 import org.apache.commons.httpclient.methods.PostMethod;
 import org.apache.commons.httpclient.methods.RequestEntity;
+import org.apache.cxf.common.xmlschema.XmlSchemaConstants;
+import org.apache.cxf.helpers.DOMUtils;
 import org.apache.cxf.helpers.IOUtils;
 import org.apache.cxf.io.CachedOutputStream;
 import org.apache.cxf.jaxrs.client.JAXRSClientFactory;
 import org.apache.cxf.jaxrs.client.WebClient;
+import org.apache.cxf.jaxrs.ext.form.Form;
 import org.apache.cxf.jaxrs.ext.xml.XMLSource;
+import org.apache.cxf.jaxrs.model.wadl.WadlGenerator;
 import org.apache.cxf.jaxrs.provider.AegisElementProvider;
+import org.apache.cxf.jaxrs.provider.JAXBElementProvider;
 import org.apache.cxf.testutil.common.AbstractBusClientServerTestBase;
 
 import org.junit.BeforeClass;
@@ -68,6 +78,88 @@ public class JAXRSClientServerSpringBookTest extends AbstractBusClientServerTest
         assertEquals("CXF", book.getName());
     }
     
+    @Test
+    public void testPostGeneratedBook() throws Exception {
+        String baseAddress = "http://localhost:" + PORT + "/the/generated/bookstore/books/1";
+        JAXBElementProvider provider = new JAXBElementProvider();
+        provider.setJaxbElementClassMap(Collections.singletonMap(
+                                          "org.apache.cxf.systest.jaxrs.codegen.schema.Book", 
+                                          "{http://superbooks}thebook"));
+        
+        WebClient wc = WebClient.create(baseAddress,
+                                        Collections.singletonList(provider));
+        wc.type("application/xml");
+        
+        org.apache.cxf.systest.jaxrs.codegen.schema.Book book = 
+            new org.apache.cxf.systest.jaxrs.codegen.schema.Book();
+        book.setId(123);
+        Response r = wc.post(book);
+        assertEquals(204, r.getStatus());
+    }
+    
+    @Test
+    public void testGetWadlFromWadlLocation() throws Exception {
+        String address = "http://localhost:" + PORT + "/the/generated";    
+        checkWadlResourcesInfo(address, address + "/bookstore", "/schemas/book.xsd", 1);
+    
+        checkSchemas(address, "/schemas/book.xsd", "/schemas/chapter.xsd", "include");
+        checkSchemas(address, "/schemas/chapter.xsd", null, null);
+    }
+    
+    @Test
+    public void testGetGeneratedWadlWithExternalSchemas() throws Exception {
+        String address = "http://localhost:" + PORT + "/the/bookstore";    
+        checkWadlResourcesInfo(address, address, "/book.xsd", 1);
+    
+        checkSchemas(address, "/book.xsd", "/bookid.xsd", "import");
+        checkSchemas(address, "/bookid.xsd", null, null);
+    }
+    
+    private void checkSchemas(String address, String schemaSegment, 
+                              String includedSchema,
+                              String refAttrName) throws Exception {
+        WebClient client = WebClient.create(address + schemaSegment);
+        WebClient.getConfig(client).getHttpConduit().getClient().setReceiveTimeout(10000000L);
+        Document doc = DOMUtils.readXml(new InputStreamReader(client.get(InputStream.class), "UTF-8"));
+        Element root = doc.getDocumentElement();
+        assertEquals(XmlSchemaConstants.XSD_NAMESPACE_URI, root.getNamespaceURI());
+        assertEquals("schema", root.getLocalName());
+        if (includedSchema != null) {
+            List<Element> includeEls = DOMUtils.getChildrenWithName(root, 
+                         XmlSchemaConstants.XSD_NAMESPACE_URI, refAttrName);
+            assertEquals(1, includeEls.size());
+            String href = includeEls.get(0).getAttribute("schemaLocation");
+            assertEquals(address + includedSchema, href);
+        }
+        
+    }
+    
+    private void checkWadlResourcesInfo(String baseURI, String requestURI, 
+                                        String schemaRef, int size) throws Exception {
+        WebClient client = WebClient.create(requestURI + "?_wadl&_type=xml");
+        Document doc = DOMUtils.readXml(new InputStreamReader(client.get(InputStream.class), "UTF-8"));
+        Element root = doc.getDocumentElement();
+        assertEquals(WadlGenerator.WADL_NS, root.getNamespaceURI());
+        assertEquals("application", root.getLocalName());
+        List<Element> grammarEls = DOMUtils.getChildrenWithName(root, 
+                                                                  WadlGenerator.WADL_NS, "grammars");
+        assertEquals(1, grammarEls.size());
+        List<Element> includeEls = DOMUtils.getChildrenWithName(grammarEls.get(0), 
+                                                                WadlGenerator.WADL_NS, "include");
+        assertEquals(1, includeEls.size());
+        String href = includeEls.get(0).getAttribute("href");
+        assertEquals(baseURI + schemaRef, href);
+        List<Element> resourcesEls = DOMUtils.getChildrenWithName(root, 
+                                                                  WadlGenerator.WADL_NS, "resources");
+        assertEquals(1, resourcesEls.size());
+        Element resourcesEl =  resourcesEls.get(0);
+        assertEquals(baseURI, resourcesEl.getAttribute("base"));
+        List<Element> resourceEls = 
+            DOMUtils.getChildrenWithName(resourcesEl, 
+                                         WadlGenerator.WADL_NS, "resource");
+        assertEquals(size, resourceEls.size());
+    }
+    
     @Test
     public void testGetBookByUriInfo() throws Exception {
         String endpointAddress =
@@ -76,6 +168,28 @@ public class JAXRSClientServerSpringBookTest extends AbstractBusClientServerTest
         getBook(endpointAddress, "resources/expected_get_book123json.txt");
     }
     
+    @Test
+    public void testGetBookWithEncodedSemicolon() throws Exception {
+        String endpointAddress =
+            "http://localhost:" + PORT + "/the/thebooks/bookstore/semicolon%3B"; 
+        WebClient client = WebClient.create(endpointAddress);
+        WebClient.getConfig(client).getHttpConduit().getClient().setReceiveTimeout(1000000);
+        Book book = client.get(Book.class);
+        assertEquals(333L, book.getId());
+        assertEquals(";", book.getName());
+    }
+    
+    @Test
+    public void testGetBookWithEncodedSemicolonAndMatrixParam() throws Exception {
+        String endpointAddress =
+            "http://localhost:" + PORT + "/the/thebooks/bookstore/semicolon2%3B;a=b"; 
+        WebClient client = WebClient.create(endpointAddress);
+        WebClient.getConfig(client).getHttpConduit().getClient().setReceiveTimeout(1000000);
+        Book book = client.get(Book.class);
+        assertEquals(333L, book.getId());
+        assertEquals(";b", book.getName());
+    }
+    
     @Test
     public void testGetBookXSLTHtml() throws Exception {
 
@@ -105,7 +219,81 @@ public class JAXRSClientServerSpringBookTest extends AbstractBusClientServerTest
             "http://localhost:" + PORT + "/the/bookstore/books/123"; 
         getBook(endpointAddress, "resources/expected_get_book123json.txt");
         getBook(endpointAddress, "resources/expected_get_book123json.txt",
-                "application/jettison");
+                "application/vnd.example-com.foo+json");
+    }
+    
+    @Test
+    public void testBookDepthExceededXML() throws Exception {
+        String endpointAddress =
+            "http://localhost:" + PORT + "/the/thebooks9/depth"; 
+        WebClient wc = WebClient.create(endpointAddress);
+        Response r = wc.post(new Book("CXF", 123L));
+        assertEquals(413, r.getStatus());
+    }
+    
+    @Test
+    public void testBookDepthExceededXMLSource() throws Exception {
+        String endpointAddress =
+            "http://localhost:" + PORT + "/the/thebooks9/depth-source"; 
+        WebClient wc = WebClient.create(endpointAddress);
+        Response r = wc.post(new Book("CXF", 123L));
+        assertEquals(413, r.getStatus());
+    }
+    
+    @Test
+    public void testBookDepthExceededXMLDom() throws Exception {
+        String endpointAddress =
+            "http://localhost:" + PORT + "/the/thebooks9/depth-dom"; 
+        WebClient wc = WebClient.create(endpointAddress);
+        WebClient.getConfig(wc).getHttpConduit().getClient().setReceiveTimeout(1000000L);
+        Response r = wc.post(new Book("CXF", 123L));
+        assertEquals(413, r.getStatus());
+    }
+    
+    @Test
+    public void testBookDepthExceededJettison() throws Exception {
+        String endpointAddress =
+            "http://localhost:" + PORT + "/the/thebooks10/depth"; 
+        WebClient wc = WebClient.create(endpointAddress);
+        wc.accept("application/json").type("application/json");
+        Response r = wc.post(new Book("CXF", 123L));
+        assertEquals(413, r.getStatus());
+    }
+    
+    @Test
+    public void testTooManyFormParams() throws Exception {
+        String endpointAddress =
+            "http://localhost:" + PORT + "/the/thebooks9/depth-form"; 
+        WebClient wc = WebClient.create(endpointAddress);
+        Response r = wc.form(new Form().set("a", "b"));
+        assertEquals(204, r.getStatus());
+        r = wc.form(new Form().set("a", "b").set("c", "b"));
+        assertEquals(413, r.getStatus());
+    }
+    
+    
+    @Test
+    public void testGetBookJsonp() throws Exception {
+        String url = "http://localhost:" + PORT + "/the/jsonp/books/123";
+        WebClient client = WebClient.create(url);
+        client.accept("application/json, application/x-javascript");
+        client.query("_jsonp", "callback");
+        Response r = client.get();
+        assertEquals("application/x-javascript", r.getMetadata().getFirst("Content-Type"));
+        assertEquals("callback({\"Book\":{\"id\":123,\"name\":\"CXF in Action\"}});",
+                     IOUtils.readStringFromStream((InputStream)r.getEntity()));
+    }
+    
+    @Test
+    public void testGetBookWithoutJsonpCallback() throws Exception {
+        String url = "http://localhost:" + PORT + "/the/jsonp/books/123";
+        WebClient client = WebClient.create(url);
+        client.accept("application/json, application/x-javascript");
+        WebClient.getConfig(client).getHttpConduit().getClient().setReceiveTimeout(1000000L);
+        Response r = client.get();
+        assertEquals("application/json", r.getMetadata().getFirst("Content-Type"));
+        assertEquals("{\"Book\":{\"id\":123,\"name\":\"CXF in Action\"}}",
+                     IOUtils.readStringFromStream((InputStream)r.getEntity()));
     }
     
     @Test
@@ -192,13 +380,13 @@ public class JAXRSClientServerSpringBookTest extends AbstractBusClientServerTest
     public void testAddInvalidXmlBook() throws Exception {
         
         doPost("http://localhost:" + PORT + "/the/bookstore/books/convert",
-               500,
+               400,
                "application/xml",
                "resources/add_book.txt",
                null);
         
         doPost("http://localhost:" + PORT + "/the/thebooks/bookstore/books/convert",
-               500,
+               400,
                "application/xml",
                "resources/add_book.txt",
                null);
@@ -209,13 +397,13 @@ public class JAXRSClientServerSpringBookTest extends AbstractBusClientServerTest
     public void testAddInvalidJsonBook() throws Exception {
         
         doPost("http://localhost:" + PORT + "/the/bookstore/books/convert",
-               500,
+               400,
                "application/json",
                "resources/add_book2json_invalid.txt",
                null);
         
         doPost("http://localhost:" + PORT + "/the/thebooks/bookstore/books/convert",
-               500,
+               400,
                "application/json",
                "resources/add_book2json_invalid.txt",
                null);
@@ -338,7 +526,7 @@ public class JAXRSClientServerSpringBookTest extends AbstractBusClientServerTest
     @Test
     public void testReaderWriterFromJaxrsFilters() throws Exception {
         String endpointAddress =
-            "http://localhost:" + PORT + "/the/thebooks5/bookstore/books/convert2/123";
+            "http://localhost:" + PORT + "/the/thebooksWithStax/bookstore/books/convert2/123";
         WebClient wc = WebClient.create(endpointAddress);
         wc.type("application/xml").accept("application/xml");
         Book2 b = new Book2();
@@ -353,7 +541,7 @@ public class JAXRSClientServerSpringBookTest extends AbstractBusClientServerTest
     @Test
     public void testReaderWriterFromInterceptors() throws Exception {
         String endpointAddress =
-            "http://localhost:" + PORT + "/the/thebooks5/bookstore/books/convert";
+            "http://localhost:" + PORT + "/the/thebooksWithStax/bookstore/books/convert";
         WebClient wc = WebClient.create(endpointAddress);
         wc.type("application/xml").accept("application/xml");
         Book2 b = new Book2();
@@ -381,7 +569,7 @@ public class JAXRSClientServerSpringBookTest extends AbstractBusClientServerTest
         
         doPost("http://localhost:" + PORT + "/the/thebooks/bookstore/books/convert",
                200,
-               "application/jettison",
+               "application/vnd.example-com.foo+json",
                "resources/add_book2json.txt",
                "resources/expected_get_book123.txt");
     }
@@ -400,7 +588,7 @@ public class JAXRSClientServerSpringBookTest extends AbstractBusClientServerTest
             int result = httpclient.executeMethod(post);
             assertEquals(expectedStatus, result);
             
-            if (expectedStatus != 500) {
+            if (expectedStatus != 400) {
                 InputStream expected = getClass().getResourceAsStream(expectedResource);
                 assertEquals(getStringFromInputStream(expected), post.getResponseBodyAsString());
             } else {
