@@ -28,6 +28,7 @@ import java.util.Arrays;
 import java.util.Collection;
 import java.util.Collections;
 import java.util.LinkedHashMap;
+import java.util.LinkedList;
 import java.util.List;
 import java.util.Map;
 
@@ -36,6 +37,7 @@ import javax.imageio.ImageIO;
 import javax.mail.util.ByteArrayDataSource;
 import javax.ws.rs.core.MediaType;
 import javax.ws.rs.core.MultivaluedMap;
+import javax.ws.rs.core.Response;
 import javax.xml.bind.JAXBContext;
 import javax.xml.bind.Unmarshaller;
 
@@ -69,7 +71,7 @@ public class JAXRSMultipartTest extends AbstractBusClientServerTestBase {
     @BeforeClass
     public static void startServers() throws Exception {
         assertTrue("server did not launch correctly",
-                   launchServer(MultipartServer.class));
+                   launchServer(MultipartServer.class, true));
     }
     
     @Test
@@ -228,6 +230,27 @@ public class JAXRSMultipartTest extends AbstractBusClientServerTestBase {
         doAddBook(address, "attachmentData2", 200);               
     }
     
+    @Test
+    public void testAddBookAsJAXBJSONProxy() throws Exception {
+        MultipartStore store = 
+            JAXRSClientFactory.create("http://localhost:" + PORT, MultipartStore.class);
+        
+        Book b = store.addBookJaxbJsonWithConsumes(new Book2("CXF in Action", 1L), 
+                                           new Book("CXF in Action - 2", 2L));
+        assertEquals(124L, b.getId());
+        assertEquals("CXF in Action - 2", b.getName());
+    }
+    
+    @Test
+    public void testAddBookAsJAXBOnlyProxy() throws Exception {
+        MultipartStore store = 
+            JAXRSClientFactory.create("http://localhost:" + PORT, MultipartStore.class);
+        
+        Book2 b = store.addBookJaxbOnlyWithConsumes(new Book2("CXF in Action", 1L));
+        assertEquals(1L, b.getId());
+        assertEquals("CXF in Action", b.getName());
+    }
+    
     @Test
     public void testAddBookAsJAXBJSONMixed() throws Exception {
         String address = "http://localhost:" + PORT + "/bookstore/books/jaxbjson";
@@ -333,12 +356,35 @@ public class JAXRSMultipartTest extends AbstractBusClientServerTestBase {
         return ImageIO.read(getClass().getResource(name));
     }
     
+    @Test
+    public void testNullableParamsMultipartAnnotation() throws Exception {
+        String address = "http://localhost:" + PORT + "/bookstore/books/testnullpart";
+        doTestNullPart(address);
+    }
+    
+    @Test
+    public void testNullableParamsFormParamAnnotation() throws Exception {
+        String address = "http://localhost:" + PORT + "/bookstore/books/testnullpartFormParam";
+        doTestNullPart(address);
+    }
+    
+    private void doTestNullPart(String address) throws Exception {
+        WebClient client = WebClient.create(address);
+        WebClient.getConfig(client).getHttpConduit().getClient().setReceiveTimeout(10000000);
+        client.type("multipart/form-data").accept("text/plain");
+        List<Attachment> atts = new LinkedList<Attachment>();
+        atts.add(new Attachment("somepart", "text/plain", "hello there"));
+        Response r = client.postCollection(atts, Attachment.class);
+        assertEquals(Response.Status.OK.getStatusCode(), r.getStatus());
+        assertEquals("nobody home", IOUtils.readStringFromStream((InputStream)r.getEntity()));
+    }
+    
     @Test
     public void testAddBookJaxbJsonImageWebClient() throws Exception {
         String address = "http://localhost:" + PORT + "/bookstore/books/jaxbjsonimage";
         WebClient client = WebClient.create(address);
         client.type("multipart/mixed").accept("multipart/mixed");
-        
+       
         Book jaxb = new Book("jaxb", 1L);
         Book json = new Book("json", 2L);
         InputStream is1 = 
