@@ -34,11 +34,13 @@ import javax.imageio.ImageIO;
 import javax.mail.util.ByteArrayDataSource;
 import javax.ws.rs.Consumes;
 import javax.ws.rs.FormParam;
+import javax.ws.rs.GET;
 import javax.ws.rs.POST;
 import javax.ws.rs.Path;
 import javax.ws.rs.Produces;
 import javax.ws.rs.WebApplicationException;
 import javax.ws.rs.core.Context;
+import javax.ws.rs.core.MediaType;
 import javax.ws.rs.core.MultivaluedMap;
 import javax.ws.rs.core.Response;
 import javax.xml.bind.JAXBContext;
@@ -141,6 +143,27 @@ public class MultipartStore {
         return new MultipartBody(newAtts);
     }
     
+    @POST
+    @Path("/books/testnullpart")
+    @Consumes("multipart/form-data")
+    @Produces("text/plain")
+    public String testNullPart(@Multipart(value = "someid", required = false) String value) {
+        if (value != null) {
+            return value;
+        } else {
+            return "nobody home";
+        }
+    }
+    
+    @POST
+    @Path("/books/testnullpartFormParam")
+    @Consumes("multipart/form-data")
+    @Produces("text/plain")
+    public String testNullPartFormParam(@FormParam(value = "someid") String value) {
+        return testNullPart(value);
+    }
+    
+    
     @POST
     @Path("/books/jaxbjsonimage")
     @Consumes("multipart/mixed")
@@ -171,6 +194,32 @@ public class MultipartStore {
         
     }
     
+    @GET
+    @Path("/books/jaxbjsonimage/read")
+    @Produces("multipart/mixed")
+    public Map<String, Book> getBookJaxbJson() throws Exception {
+        Book jaxb = new Book("jaxb", 1L);
+        Book json = new Book("json", 2L);
+        Map<String, Book> objects = new LinkedHashMap<String, Book>();
+        objects.put(MediaType.APPLICATION_XML, jaxb);
+        objects.put(MediaType.APPLICATION_JSON, json);
+        return objects;
+        
+    }
+    
+    @GET
+    @Path("/books/jaxbjsonimage/read-object")
+    @Produces("multipart/mixed")
+    public Map<String, Object> getBookJaxbJsonObject() throws Exception {
+        Book jaxb = new Book("jaxb", 1L);
+        Book json = new Book("json", 2L);
+        Map<String, Object> objects = new LinkedHashMap<String, Object>();
+        objects.put(MediaType.APPLICATION_XML, jaxb);
+        objects.put(MediaType.APPLICATION_JSON, json);
+        return objects;
+        
+    }
+    
     @POST
     @Path("/books/stream")
     @Produces("text/xml")
@@ -295,10 +344,40 @@ public class MultipartStore {
         return books2;
     }
     
+    @POST
+    @Path("/books/jaxbonly")
+    @Consumes("multipart/mixed;type=text/xml")
+    @Produces("multipart/mixed;type=text/xml")
+    public List<Book> addBooksWithoutHeader(List<Book> books) {
+        return addBooks(books);
+    }
+    
+    @POST
+    @Path("/books/jaxbjsonconsumes")
+    @Consumes("multipart/related")
+    @Produces("text/xml")
+    public Book addBookJaxbJsonWithConsumes(
+        @Multipart(value = "rootPart", type = "text/xml") Book2 b1,
+        @Multipart(value = "book2", type = "application/json") Book b2) throws Exception {
+        return addBookJaxbJson(b1, b2);    
+    }
+    
+    @POST
+    @Path("/books/jaxbonly")
+    @Consumes("multipart/related")
+    @Produces("text/xml")
+    public Book2 addBookJaxbOnlyWithConsumes(
+        @Multipart(value = "rootPart", type = "text/xml") Book2 b1) throws Exception {
+        if (!"CXF in Action".equals(b1.getName())) {
+            throw new WebApplicationException();
+        }
+        return b1;    
+    }
+    
     @POST
     @Path("/books/jaxbjson")
     @Produces("text/xml")
-    public Response addBookJaxbJson(
+    public Book addBookJaxbJson(
         @Multipart(value = "rootPart", type = "text/xml") Book2 b1,
         @Multipart(value = "book2", type = "application/json") Book b2) 
         throws Exception {
@@ -307,7 +386,7 @@ public class MultipartStore {
             throw new WebApplicationException();
         }
         b2.setId(124);
-        return Response.ok(b2).build();
+        return b2;
     }
     
     @POST
@@ -342,6 +421,23 @@ public class MultipartStore {
         return Response.ok(b1).build();
     }
     
+    @POST
+    @Path("/books/fileform")
+    @Produces("text/xml")
+    @Consumes("multipart/form-data")
+    public Response addBookFilesForm(MultipartBody body) 
+        throws Exception {
+        String owner = body.getAttachmentObject("owner", String.class);
+        Book book = body.getAttachmentObject("file", Book.class);
+        if (!"CXF in Action - 1".equals(book.getName())
+            || !"Larry".equals(owner)) {
+            throw new WebApplicationException();
+        }
+        book.setId(124);
+        book.setName("CXF in Action - 2");
+        return Response.ok(book).build();
+    }
+    
     @POST
     @Path("/books/jaxbform")
     @Produces("text/xml")
