@@ -21,6 +21,7 @@ package org.apache.cxf.tools.wsdlto.jaxws;
 import java.io.File;
 import java.io.ObjectStreamClass;
 import java.lang.annotation.Annotation;
+import java.lang.reflect.Field;
 import java.lang.reflect.Method;
 import java.lang.reflect.Modifier;
 import java.lang.reflect.ParameterizedType;
@@ -322,6 +323,45 @@ public class CodeGenTest extends AbstractCodeGenTest {
         assertEquals(method2.getName() + "()" + " Annotation : WebMethod.operationName ", "greetMeSometime",
                      webMethodAnno2.operationName());
 
+    }
+    @Test
+    public void testAsyncMethodFromCommandLine() throws Exception {
+        env.put(ToolConstants.CFG_WSDLURL, getLocation("/wsdl2java_wsdl/hello_world.wsdl"));
+        env.put(ToolConstants.CFG_ASYNCMETHODS, new String[0]);
+        processor.setContext(env);
+        processor.execute();
+
+        assertNotNull(output);
+
+        File org = new File(output, "org");
+        assertTrue(org.exists());
+        File apache = new File(org, "apache");
+        assertTrue(apache.exists());
+        File cxf = new File(apache, "cxf");
+        assertTrue(cxf.exists());
+        File w2j = new File(cxf, "w2j");
+        assertTrue(w2j.exists());
+        File async = new File(w2j, "hello_world_soap_http");
+        assertTrue(async.exists());
+
+        File[] files = async.listFiles();
+        assertEquals(9, files.length);
+
+        Class<?> clz = classLoader.loadClass("org.apache.cxf.w2j.hello_world_soap_http.Greeter");
+
+        Method method1 = clz.getMethod("greetMeSometimeAsync", new Class[] {java.lang.String.class,
+                                                                            javax.xml.ws.AsyncHandler.class});
+        WebMethod webMethodAnno1 = AnnotationUtil.getPrivMethodAnnotation(method1, WebMethod.class);
+
+        assertEquals(method1.getName() + "()" + " Annotation : WebMethod.operationName ", "greetMeSometime",
+                     webMethodAnno1.operationName());
+
+        java.lang.reflect.Method method2 = clz.getMethod("greetMeSometimeAsync",
+                                                         new Class[] {java.lang.String.class});
+        WebMethod webMethodAnno2 = AnnotationUtil.getPrivMethodAnnotation(method2, WebMethod.class);
+        assertEquals(method2.getName() + "()" + " Annotation : WebMethod.operationName ", "greetMeSometime",
+                     webMethodAnno2.operationName());
+
     }
 
     @Test
@@ -1228,7 +1268,37 @@ public class CodeGenTest extends AbstractCodeGenTest {
         assertTrue(contents.indexOf("SOAPBinding.ParameterStyle.BARE") != -1);
         assertTrue(contents.indexOf("@ResponseWrapper") == -1);
     }
+    @Test
+    public void testBareFromCommandLine() throws Exception {
+        env.put(ToolConstants.CFG_WSDLURL, getLocation("/wsdl2java_wsdl/hello_world.wsdl"));
+        env.put(ToolConstants.CFG_BAREMETHODS, new String[0]);
 
+        processor.setContext(env);
+        processor.execute();
+
+        File greeter = new File(output, "org/apache/cxf/w2j/hello_world_soap_http/Greeter.java");
+        assertTrue(output.exists());
+        String contents = FileUtils.getStringFromFile(greeter);
+        assertTrue(contents.indexOf("SOAPBinding.ParameterStyle.BARE") != -1);
+        assertTrue(contents.indexOf("@ResponseWrapper") == -1);
+    }
+    @Test
+    public void testMimeFromCommandLine() throws Exception {
+        env.put(ToolConstants.CFG_WSDLURL, getLocation("/wsdl2java_wsdl/hello-mime.wsdl"));
+        env.put(ToolConstants.CFG_MIMEMETHODS, new String[0]);
+
+        processor.setContext(env);
+        processor.execute();
+
+        String str1 = "javax.xml.ws.Holder<java.awt.Image>";
+        String str2 = "javax.xml.transform.Source";
+
+        String file = getStringFromFile(new File(output.getCanonicalPath()
+                                        + "/org/apache/cxf/w2j/hello_world_mime/Hello.java"));
+
+        assertTrue(file.contains(str1));
+        assertTrue(file.contains(str2));
+    }
     @Test
     public void testXmlSeeAlso() throws Exception {
         env.put(ToolConstants.CFG_WSDLURL, getLocation("/wsdl2java_wsdl/cardealer.wsdl"));
@@ -1338,7 +1408,7 @@ public class CodeGenTest extends AbstractCodeGenTest {
     @Test
     public void testCXFNotType() throws Exception {
         env.put(ToolConstants.CFG_WSDLURL, getLocation("/wsdl2java_wsdl/hello_world.wsdl"));
-        env.put(ToolConstants.CFG_USE_FQCN_FAULT_SERIAL_VERSION_UID, Boolean.TRUE);
+        env.put(ToolConstants.CFG_FAULT_SERIAL_VERSION_UID, "FQCN");
         processor.setContext(env);
         processor.execute();
         
@@ -1351,9 +1421,9 @@ public class CodeGenTest extends AbstractCodeGenTest {
     }
 
     @Test
-    public void testCXF2808() throws Exception {
+    public void testFaultSerialVersionUIDNONEFQCN() throws Exception {
         env.put(ToolConstants.CFG_WSDLURL, getLocation("/wsdl2java_wsdl/hello_world.wsdl"));
-        env.put(ToolConstants.CFG_USE_FQCN_FAULT_SERIAL_VERSION_UID, Boolean.TRUE);
+        env.put(ToolConstants.CFG_FAULT_SERIAL_VERSION_UID, "FQCN");
         processor.setContext(env);
         processor.execute();
 
@@ -1363,12 +1433,144 @@ public class CodeGenTest extends AbstractCodeGenTest {
         assertTrue(faultFile.exists());
 
         Class<?> fault = classLoader.loadClass("org.apache.cxf.w2j.hello_world_soap_http.NoSuchCodeLitFault");
+        Field serialVersionUID = fault.getDeclaredField("serialVersionUID");
+        assertNotNull(serialVersionUID);
         assertEquals(fault.getName().hashCode(), ObjectStreamClass.lookup(fault).getSerialVersionUID());
         fault = classLoader.loadClass("org.apache.cxf.w2j.hello_world_soap_http.BadRecordLitFault");
         assertEquals(fault.getName().hashCode(), ObjectStreamClass.lookup(fault).getSerialVersionUID());
     }
     
     
+    @Test
+    public void testNoFaultSerialVersionUID() throws Exception {
+        env.put(ToolConstants.CFG_WSDLURL, getLocation("/wsdl2java_wsdl/hello_world.wsdl"));
+        processor.setContext(env);
+        processor.execute();
+
+        File faultFile = new File(output, "org/apache/cxf/w2j/hello_world_soap_http/NoSuchCodeLitFault.java");
+        assertTrue(faultFile.exists());
+        faultFile = new File(output, "org/apache/cxf/w2j/hello_world_soap_http/BadRecordLitFault.java");
+        assertTrue(faultFile.exists());
+
+        Class<?> fault = classLoader.loadClass("org.apache.cxf.w2j.hello_world_soap_http.NoSuchCodeLitFault");
+        try {
+            fault.getDeclaredField("serialVersionUID");
+            fail("shouldn't have serialVersionUID field");
+        } catch (NoSuchFieldException e) {
+            //
+        }
+        fault = classLoader.loadClass("org.apache.cxf.w2j.hello_world_soap_http.BadRecordLitFault");
+        try {
+            fault.getDeclaredField("serialVersionUID");
+            fail("shouldn't have serialVersionUID field");
+        } catch (NoSuchFieldException e) {
+            //
+        }
+                
+    }
+    
+    @Test
+    public void testFaultSerialVersionUIDNONE() throws Exception {
+        env.put(ToolConstants.CFG_WSDLURL, getLocation("/wsdl2java_wsdl/hello_world.wsdl"));
+        env.put(ToolConstants.CFG_FAULT_SERIAL_VERSION_UID, "NONE");
+        processor.setContext(env);
+        processor.execute();
+
+        File faultFile = new File(output, "org/apache/cxf/w2j/hello_world_soap_http/NoSuchCodeLitFault.java");
+        assertTrue(faultFile.exists());
+        faultFile = new File(output, "org/apache/cxf/w2j/hello_world_soap_http/BadRecordLitFault.java");
+        assertTrue(faultFile.exists());
+
+        Class<?> fault = classLoader.loadClass("org.apache.cxf.w2j.hello_world_soap_http.NoSuchCodeLitFault");
+        try {
+            fault.getDeclaredField("serialVersionUID");
+            fail("shouldn't have serialVersionUID field");
+        } catch (NoSuchFieldException e) {
+            //
+        }
+        fault = classLoader.loadClass("org.apache.cxf.w2j.hello_world_soap_http.BadRecordLitFault");
+        try {
+            fault.getDeclaredField("serialVersionUID");
+            fail("shouldn't have serialVersionUID field");
+        } catch (NoSuchFieldException e) {
+            //
+        }
+            
+    }
+    
+    @Test
+    public void testFaultSerialVersionUIDNONETIMESTAMP() throws Exception {
+        env.put(ToolConstants.CFG_WSDLURL, getLocation("/wsdl2java_wsdl/hello_world.wsdl"));
+        env.put(ToolConstants.CFG_FAULT_SERIAL_VERSION_UID, "TIMESTAMP");
+        processor.setContext(env);
+        processor.execute();
+
+        File faultFile = new File(output, "org/apache/cxf/w2j/hello_world_soap_http/NoSuchCodeLitFault.java");
+        assertTrue(faultFile.exists());
+        faultFile = new File(output, "org/apache/cxf/w2j/hello_world_soap_http/BadRecordLitFault.java");
+        assertTrue(faultFile.exists());
+
+        Class<?> fault = classLoader.loadClass("org.apache.cxf.w2j.hello_world_soap_http.NoSuchCodeLitFault");
+        Field serialVersionUID = fault.getDeclaredField("serialVersionUID");
+        assertNotNull(serialVersionUID);
+        
+        fault = classLoader.loadClass("org.apache.cxf.w2j.hello_world_soap_http.BadRecordLitFault");
+        serialVersionUID = fault.getDeclaredField("serialVersionUID");
+        assertNotNull(serialVersionUID);
+    }
+    @Test
+    public void testFaultSerialVersionUIDNumber() throws Exception {
+        env.put(ToolConstants.CFG_WSDLURL, getLocation("/wsdl2java_wsdl/hello_world.wsdl"));
+        env.put(ToolConstants.CFG_FAULT_SERIAL_VERSION_UID, "123456789");
+        processor.setContext(env);
+        processor.execute();
+
+        File faultFile = new File(output, "org/apache/cxf/w2j/hello_world_soap_http/NoSuchCodeLitFault.java");
+        assertTrue(faultFile.exists());
+        faultFile = new File(output, "org/apache/cxf/w2j/hello_world_soap_http/BadRecordLitFault.java");
+        assertTrue(faultFile.exists());
+
+        Class<?> fault = classLoader.loadClass("org.apache.cxf.w2j.hello_world_soap_http.NoSuchCodeLitFault");
+        Field serialVersionUID = fault.getDeclaredField("serialVersionUID");
+        assertNotNull(serialVersionUID);
+        Long l = (Long)serialVersionUID.get(null);
+        assertEquals(123456789L, l.longValue());
+        
+        fault = classLoader.loadClass("org.apache.cxf.w2j.hello_world_soap_http.BadRecordLitFault");
+        serialVersionUID = fault.getDeclaredField("serialVersionUID");
+        assertNotNull(serialVersionUID);
+        assertEquals(123456789L, l.longValue());
+    }
+    @Test
+    public void testNoExceptionSuper() throws Exception {
+        env.put(ToolConstants.CFG_WSDLURL, getLocation("/wsdl2java_wsdl/hello_world.wsdl"));
+        processor.setContext(env);
+        processor.execute();
+        
+        File faultFile = new File(output, "org/apache/cxf/w2j/hello_world_soap_http/NoSuchCodeLitFault.java");
+        assertTrue(faultFile.exists());
+        faultFile = new File(output, "org/apache/cxf/w2j/hello_world_soap_http/BadRecordLitFault.java");
+        assertTrue(faultFile.exists());
+        
+        Class<?> fault = classLoader.loadClass("org.apache.cxf.w2j.hello_world_soap_http.NoSuchCodeLitFault");
+        assertEquals(Exception.class, fault.getSuperclass());
+    }
+    @Test
+    public void testExceptionSuper() throws Exception {
+        env.put(ToolConstants.CFG_WSDLURL, getLocation("/wsdl2java_wsdl/hello_world.wsdl"));
+        env.put(ToolConstants.CFG_EXCEPTION_SUPER, "java.lang.RuntimeException");
+        processor.setContext(env);
+        processor.execute();
+        
+        File faultFile = new File(output, "org/apache/cxf/w2j/hello_world_soap_http/NoSuchCodeLitFault.java");
+        assertTrue(faultFile.exists());
+        faultFile = new File(output, "org/apache/cxf/w2j/hello_world_soap_http/BadRecordLitFault.java");
+        assertTrue(faultFile.exists());
+        
+        Class<?> fault = classLoader.loadClass("org.apache.cxf.w2j.hello_world_soap_http.NoSuchCodeLitFault");
+        assertEquals(RuntimeException.class, fault.getSuperclass());
+    }
+
     @Test
     public void testExtensionWrapper() throws Exception {
         env.put(ToolConstants.CFG_WSDLURL,
@@ -1397,7 +1599,6 @@ public class CodeGenTest extends AbstractCodeGenTest {
         List<String> results1 = FileUtils.readLines(new File(output.getCanonicalPath(),
             "org/apache/cxf/w2j/hello_world_soap_http/Greeter.java"));
 
-        System.out.println(results1);
         assertTrue(results1.contains(" * porttype documentation"));
         assertTrue(results1.contains("     * porttype op documentation"));
 
@@ -1424,4 +1625,18 @@ public class CodeGenTest extends AbstractCodeGenTest {
 
     }
 
+    
+    @Test
+    // CXF-4135
+    public void testPassShemaFileAsJAXBBinding() throws Exception {
+        env.put(ToolConstants.CFG_WSDLURL, getLocation("/wsdl2java_wsdl/hello_world.wsdl"));
+        env.put(ToolConstants.CFG_BINDING, getLocation("/wsdl2java_wsdl/Invoice.xsd"));
+        processor.setContext(env);
+        processor.execute();
+        File generateFromBindingFile = new File(output, "org/apache/invoice/Invoice.java");
+        assertTrue(generateFromBindingFile.exists());
+        File seiFile = new File(output, "org/apache/cxf/w2j/hello_world_soap_http/Greeter.java");
+        assertTrue(seiFile.exists());
+    }
+    
 }
